/**************************************************************
 *                                                            *
 * description: tools for use with OpenGL                     *
 * version    : 2.5c                                          *
 * date       : 21.Feb.2003                                   *
 * modified   : 03.Mar.2005                                   *
 * author     : Jens Krger (with help form various sources)  *
 * e-mail     : jens.kruger@in.tum.de                         *
 *                                                            *
 **************************************************************/
#include "openglUtils.h"


int openGLUtils::log2(int n) {
	int iLog=0;
	while( n>>=1 ) iLog++;
	return iLog;
}

void openGLUtils::normalize4(float v[4]) {
  float d = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
  v[0] /= d;
  v[1] /= d;
  v[2] /= d;
}

void openGLUtils::normalize3(float v[3]) {
  float d = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
  v[0] /= d;
  v[1] /= d;
  v[2] /= d;
}

TIMER_VAR(sys_timer);

void openGLUtils::fpsTimerStart() {
	TIMER_START(sys_timer);    // start timer
	TIMER_PROBE(sys_timer);
	last=TIMER_RESULT(sys_timer);
}

void openGLUtils::fpsTimerStartFrame() {	
	TIMER_PROBE(sys_timer);               // Query timer
	elapsed_time=TIMER_RESULT(sys_timer);
}

void openGLUtils::fpsTimerGetFPS(double *_fps) {	
	if (elapsed_time-last>=1.0) {
		fpsCounter=fps/(elapsed_time-last);		

		*_fps = fpsCounter;
		
		last=elapsed_time;
		fps=0.0;		
	}
	fps+=1.0; 
}

double openGLUtils::fpsTimerShowFPS(const char *progName) {	
	if (elapsed_time-last>=1.0) {
		fpsCounter=fps/(elapsed_time-last);		

		char text[255];	
		sprintf(text,"%s @ %.2f fps",progName,fpsCounter);
		glutSetWindowTitle(text);
		
		last=elapsed_time;
		fps=0.0;

	}
	fps+=1.0; 
	return fpsCounter;
}

inline float openGLUtils::smoothstep(float x) {return 3*x*x-2*x*x*x;}
inline float openGLUtils::scalarProd3(float a[3], float b[3]) {return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];}
inline float openGLUtils::scalarProd4(float a[4], float b[4]) {return a[0]*b[0]+a[1]*b[1]+a[2]*b[2]+a[3]*b[3];}

// attention: this function computes A=B*A   (NOT A=A*B)
void openGLUtils::MatrixMatrixProd(float a[16], float b[16]) {
	float c[16];
	for (int x = 0;x<16;x+=4)
		for (int y = 0;y<4;y++) 
			c[x+y] = b[0+y]*a[0+x]+b[4+y]*a[1+x]+b[8+y]*a[2+x]+b[12+y]*a[3+x];

	for (int i=0;i<16;i++) a[i] = c[i];	
}

inline void openGLUtils::MatrixVectorProd3(float M[16], double v[3]) {
	double a=M[ 0]*v[0]+M[ 4]*v[1]+M[ 8]*v[2];
	double b=M[ 1]*v[0]+M[ 5]*v[1]+M[ 9]*v[2];
	v[2]=M[ 2]*v[0]+M[ 6]*v[1]+M[10]*v[2];       
	v[0]=a;
	v[1]=b;       
}

inline void openGLUtils::MatrixVectorProd3(float M[16], float v[3]) {
	float a=M[ 0]*v[0]+M[ 4]*v[1]+M[ 8]*v[2];
	float b=M[ 1]*v[0]+M[ 5]*v[1]+M[ 9]*v[2];
	v[2]=M[ 2]*v[0]+M[ 6]*v[1]+M[10]*v[2];       
	v[0]=a;
	v[1]=b;       
}

inline void openGLUtils::MatrixVectorProd4(float M[16], double v[4]) {
	double a=M[ 0]*v[0]+M[ 4]*v[1]+M[ 8]*v[2]+M[12]*v[3];
	double b=M[ 1]*v[0]+M[ 5]*v[1]+M[ 9]*v[2]+M[13]*v[3];
	double c=M[ 2]*v[0]+M[ 6]*v[1]+M[10]*v[2]+M[14]*v[3];
	v[3]=M[ 3]*v[0]+M[ 7]*v[1]+M[11]*v[2]+M[15]*v[3];
	v[0]=a;
	v[1]=b;
	v[2]=c;
}

inline void openGLUtils::MatrixVectorProd4(float M[16], float v[4]) {
	float a=M[ 0]*v[0]+M[ 4]*v[1]+M[ 8]*v[2]+M[12]*v[3];
	float b=M[ 1]*v[0]+M[ 5]*v[1]+M[ 9]*v[2]+M[13]*v[3];
	float c=M[ 2]*v[0]+M[ 6]*v[1]+M[10]*v[2]+M[14]*v[3];
	v[3]=M[ 3]*v[0]+M[ 7]*v[1]+M[11]*v[2]+M[15]*v[3];
	v[0]=a;
	v[1]=b;
	v[2]=c;
}

inline void openGLUtils::printVector4(float v[4]) {printf("[%g,%g,%g,%g]",v[0],v[1],v[2],v[3]);}
inline void openGLUtils::printVector3(float v[3]) {printf("[%g,%g,%g]",v[0],v[1],v[2]);}

inline void openGLUtils::printMatrix4(float m[16]) {
	printf("[%g,\t%g,\t%g,\t%g]\n \
			[%g,\t%g,\t%g,\t%g]\n \
			[%g,\t%g,\t%g,\t%g]\n \
			[%g,\t%g,\t%g,\t%g]\n",m[0],m[4], m[8],m[12],
								m[1],m[5], m[9],m[13],
								m[2],m[6],m[10],m[14],
								m[3],m[7],m[11],m[15]);
}

void openGLUtils::glPrintString(void *font, char *string) {
	int len = (int)strlen(string),i;
	glRasterPos3f(len*-0.02 ,1,-2);  
	for(i=0;i<len;i++) glutBitmapCharacter(font,string[i]);
}

inline void openGLUtils::setVec3(float a[3], float b[3]) {
	a[0] = b[0];
	a[1] = b[1];
	a[2] = b[2];
}

void openGLUtils::swapVec3(float a[3], float b[3]) {
	float c[3];
	setVec3(c,a);
	setVec3(a,b);
	setVec3(b,c);
}

inline bool openGLUtils::floatEqual(float a, float b) {
	return fabs(a-b) < 0.0000000000001;
}


/*
 * EULER TRANSFORMATION, using Quaternions
 *
 * You need not to understand the math, treat it a "black box"
 * that performs a rotation about head (y-axis), pitch (x-axis) and roll (-z-axis)
 */
void openGLUtils::MultEulerf(float head, float pitch, float roll) {
    float SINh=SIN(0.5f*head);
    float SINp=SIN(0.5f*pitch);
    float SINr=SIN(0.5f*roll);
    float COSh=COS(0.5f*head);
    float COSp=COS(0.5f*pitch);
    float COSr=COS(0.5f*roll);
    // The Euler-Quaternion (already concatenated)
    float  qx = SINh*COSp*SINr+COSh*SINp*COSr;
    float  qy =-COSh*SINp*SINr+SINh*COSp*COSr;
    float  qz =-SINh*SINp*COSr+COSh*COSp*SINr;
    float  qw = SINh*SINp*SINr+COSh*COSp*COSr;
    // Build the Matrix from it
    float M[16]= {
      1-2*(qy*qy+qz*qz),   2*(qx*qy+qw*qz),   2*(qx*qz-qw*qy), 0.0f,
        2*(qx*qy-qw*qz), 1-2*(qx*qx+qz*qz),   2*(qy*qz+qw*qx), 0.0f,
        2*(qx*qz+qw*qy),   2*(qy*qz-qw*qx), 1-2*(qx*qx+qy*qy), 0.0f,
        0.0f,              0.0f,              0.0f,            1.0f
    };

    glMultMatrixf(M);
}


/*
 * INVERT MODELVIEW 
 * Works only with translates & rotates !
 */
void openGLUtils::GetInvertedModelview(float invM[16]) {
	float M[16];
	glGetFloatv(GL_MODELVIEW_MATRIX,M);
	// Let R=upper left 3x3. R is orthonormal (rotation-matrix) <=> R^-1=R^T.
	invM[ 0]=M[ 0];   invM[ 4]=M[ 1];   invM[ 8]=M[ 2];
	invM[ 1]=M[ 4];   invM[ 5]=M[ 5];   invM[ 9]=M[ 6];
	invM[ 2]=M[ 8];   invM[ 6]=M[ 9];   invM[10]=M[10];
	// Let v=upper right 3D-vector. v^-1=-R^T*v. v is the translation vector.
	invM[12]=-invM[ 0]*M[12]-invM[ 4]*M[13]-invM[ 8]*M[14];
	invM[13]=-invM[ 1]*M[12]-invM[ 5]*M[13]-invM[ 9]*M[14];
	invM[14]=-invM[ 2]*M[12]-invM[ 6]*M[13]-invM[10]*M[14];
	// rest is just like ID
	invM[ 3]=0.0f;   invM[ 7]=0.0f;   invM[11]=0.0f;   invM[15]=1.0f;
}

inline void openGLUtils::findMinMax(int a, int b, int &min, int &max) {
	if (a<b) {min = a; max = b;} else {min = b; max = a;}
}

inline void openGLUtils::findMinMax(int a, int b, int c, int &min, int &max) {
	int min1, max1, dummy;
	findMinMax(a,b,min1,max1);
	findMinMax(min1,c,min,dummy);
	findMinMax(max1,c,dummy,max);
}

inline void openGLUtils::findMinMax(int a, int b, int c, int d, int &min, int &max) {
	int min1, min2, max1, max2, dummy;
	findMinMax(a,b,min1,max1);
	findMinMax(c,d,min2,max2);
	findMinMax(min1,min2,min,dummy);
	findMinMax(max1,max2,dummy,max);
}


void openGLUtils::saveData2BMP(char *fileName, GLubyte *image, int width, int height, bool BGR) {
	FILE *stream;
	printf("%i x %i\n",width,height);
	if((stream = fopen(fileName, "wb"))==NULL)
		printf("Error while saving data into file %s.\n",fileName);
	else {
		// ************* write BMP-Header
		fprintf(stream,"BM");               // all BMP-Files start with "BM"
		unsigned int header[3];
		int iPad=((width%4) == 0 ? 0 : 4-(width%4));
		header[0] = 54+(width+iPad)*height*3;		// filesize = 54 (header) + size * size *channels
		header[1] = 0;						// reserved = 0 (4 Bytes)
		header[2] = 54;						// File offset to Raster Data
		fwrite(header, 4, 3, stream);
		// ************* write BMP-Info-Header
		unsigned int infoHeader[10];
		infoHeader[0] = 40;	                // size of info header 
		infoHeader[1] = width;              // Bitmap Width
		infoHeader[2] = height;             // Bitmap Height
		infoHeader[3] = 1+65536*8*3;        // first 2 bytes=Number of Planes (=1)
											// next  2 bytes=BPP
		infoHeader[4] = 0;					// compression (0 = none)
		infoHeader[5] = 0;					// compressed file size (0 if no compression)
		infoHeader[6] = 11810;				// horizontal resolution: Pixels/meter (11810 = 300 dpi)
		infoHeader[7] = 11810;				// vertical resolution: Pixels/meter (11810 = 300 dpi)
		infoHeader[8] = 0;					// Number of actually used colors
		infoHeader[9] = 0;					// Number of important colors  0 = all		
		fwrite(infoHeader, 4, 10, stream);
		// for some reason the data in BMP is stored BGR, so convert RGB to BGR, if necessary
		GLubyte *saveImage;
		if (BGR) {
			saveImage=image;			
		}
		else{ // RGB to BGR
			saveImage = new GLubyte[width*height*3];
			for (int i = 0; i < width*height*3; i += 3) {
				saveImage[i]   = image[i+2];
				saveImage[i+1] = image[i+1];
				saveImage[i+2] = image[i];
			}			
		}
		// write data
		if (iPad==0) {
			fwrite(saveImage,3*sizeof(GLubyte),width*height,stream);
		}
		else {
			GLubyte c[9]={0,0,0,0,0,0,0,0,0};
			for (int i=0; i<height; i++) {
				fwrite(&saveImage[i*3*width],3*sizeof(GLubyte),width,stream);
				fwrite(c,3*sizeof(GLubyte),iPad,stream);
			}
		}
		fflush(stream);
		fclose( stream );
		if (saveImage!=image && saveImage) delete[] saveImage;
	}
}


void openGLUtils::findNextFileName(char *prefix, char *extension, char** fileName) {
	int index=0;
	FILE *stream;	
	*fileName = new char[100];	
	do {
		sprintf(*fileName,"%s%i.%s",prefix,index++,extension);
		stream = fopen(*fileName, "r");
		if (stream != NULL) fclose( stream );
	} while (stream!=NULL);
}

void openGLUtils::saveWindow(char *fileName) {
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);

	GLubyte *image = new GLubyte[viewport[2]*viewport[3]*3];
	glPixelStorei(GL_PACK_ALIGNMENT, 1);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glReadPixels(0,0,viewport[2],viewport[3],GL_RGB,GL_UNSIGNED_BYTE,image);
	saveData2BMP(fileName, image, viewport[2], viewport[3]);
	delete[] image;
}

void openGLUtils::saveWindow() {
	char *fileName;
	findNextFileName("image", "bmp", &fileName);
	saveWindow(fileName);
	delete[] fileName;
}

void openGLUtils::rotate(float &rotx, float &roty, float &rotz, float rotMatrix[16], bool fluid, float viscosity) {
	float M[16];

	// rotate around X-axis
	float rotSin = SIN(-roty);
	float rotCos = COS(-roty);

	M[0]  =	1;        M[4] = 0;        M[8]  = 0;       M[12] = 0;
	M[1]  = 0;        M[5] = rotCos;   M[9]  = rotSin;  M[13] = 0;
	M[2]  = 0;	      M[6] = -rotSin;  M[10] = rotCos;  M[14] = 0;
	M[3]  = 0;        M[7] = 0;        M[11] = 0;	    M[15] = 1;
	MatrixMatrixProd(rotMatrix, M);

	// rotate around Y-axis
	rotSin = SIN(rotx);
	rotCos = COS(rotx);

	M[0]  =	rotCos;   M[4]  = 0;       M[8]  = rotSin;  M[12] = 0;
	M[1]  = 0;        M[5]  = 1;       M[9]  = 0;       M[13] = 0;
	M[2]  = -rotSin;  M[6]  = 0;       M[10] = rotCos;  M[14] = 0;
	M[3]  = 0;        M[7]  = 0;       M[11] = 0;	    M[15] = 1;
	MatrixMatrixProd(rotMatrix, M);

	// rotate around Z-axis
	rotSin = SIN(-rotz);
	rotCos = COS(-rotz);

	M[0] =  rotCos;   M[4]  = rotSin;  M[8]  = 0;       M[3]  = 0;
	M[1] = -rotSin;   M[5]  = rotCos;  M[9]  = 0;       M[7]  = 0;
	M[2] = 0;	      M[6]  = 0;       M[10] = 1;       M[11] = 0;
	M[3] = 0;         M[7]  = 0;       M[11] = 0;       M[15] = 1;
	MatrixMatrixProd(rotMatrix, M);

	if (fluid) {
		rotz /= viscosity;
		rotx /= viscosity;
		roty /= viscosity;
	} else {
		rotz = 0;
		rotx = 0;
		roty = 0;
	}

	glMatrixMode(GL_MODELVIEW);
	glMultMatrixf(rotMatrix);
}

void openGLUtils::rotate(float &rotx, float &roty, float &rotz, FLOATMATRIX4 &rotMatrix, const bool fluid, const float viscosity) {
	FLOATMATRIX4 M;

	// rotate around X-axis
	M.RotationX(-roty*0.017453292);
	rotMatrix = M*rotMatrix;

	// rotate around Y-axis
	M.RotationY(rotx*0.017453292);
	rotMatrix = M*rotMatrix;

	// rotate around Z-axis
	M.RotationZ(-rotz*0.017453292);
	rotMatrix = M*rotMatrix;

	if (fluid) {
		rotz /= viscosity;
		rotx /= viscosity;
		roty /= viscosity;
	} else {
		rotz = 0;
		rotx = 0;
		roty = 0;
	}

	rotMatrix.multModelview();
}


bool openGLUtils::isExtensionSupported(const char *extension) {
  const GLubyte *extensions = NULL;
  const GLubyte *start;
  GLubyte *where, *terminator;

  /* Extension names should not have spaces. */
  where = (GLubyte *) strchr(extension, ' ');
  if (where || *extension == '\0')
    return false;

  extensions = glGetString(GL_EXTENSIONS);

  /* It takes a bit of care to be fool-proof about parsing the
     OpenGL extensions string.  Don't be fooled by sub-strings,
     etc. */
  start = extensions;
  for (;;) {
    where = (GLubyte *) strstr((const char *) start, extension);
    if (!where)
      break;
    terminator = where + strlen(extension);
    if (where == start || *(where - 1) == ' ')
      if (*terminator == ' ' || *terminator == '\0')
        return true;
    start = terminator;
  }

  return false;
}

void openGLUtils::exitWithError(char *string, int exitCode) {
	fprintf(stderr,string);
	fprintf(stderr,"\nPress return to exit...");
	char dummy;	scanf( "%c",&dummy);
	exit(exitCode);
}


bool openGLUtils::fileExits(const char* filename) {
	FILE *file = fopen(filename,"rb");
	if (file != NULL) {
		fclose(file);
		return true;
	} else return false;
}