/************************************************************
 *															*
 * decr     : base class for al clClasses					*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 18.05.2005									*
 * modified	: 18.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#include "clClass.h"

int clClass::ms_pFBOID=0;
GLuint clClass::ms_pFBO=NULL;
char* clClass::ms_vsPassThrough = "\
\
void main(void)\
{\
   gl_Position       = vec4( gl_Vertex.xy, 0.0, 1.0 );\
   gl_TexCoord[0].xy = gl_MultiTexCoord0.xy;\
}";

clMemMan* clClass::ms_memoryMananger=NULL;

bool clClass::StartupCLFrameWork() {
	// **** init GLEW ****
	glewExperimental = GL_TRUE;
	int err = glewInit();
	if (err != GLEW_OK) {
		VERB("Error initializing GLEW ");VERBST(glewGetErrorString(err));VERBLN;
		return false;
	}

	// check for FBOs
	if (!glewGetExtension("GL_EXT_framebuffer_object")) {
		VERB("GL_EXT_framebuffer_object not supported.\n");
		return false;
	}

	// check for Shader Objects
	if (!glewGetExtension("GL_ARB_shader_objects")) {
		VERB("ARB_shader_objects not supported!\n");
		return false;
	}

	// check for GLSL
	if (!glewGetExtension("GL_ARB_shading_language_100")) {
		VERB("ARB_shading_language_100 not supported!\n");
		return false;
	}

	// create memory manager
	if (ms_memoryMananger == NULL) ms_memoryMananger = new clMemMan();

	// create the global FBO
	if (ms_pFBO == NULL)
		ms_pFBOID = ms_memoryMananger->getFBO(ms_pFBO);

	return true;
}

void clClass::ShutdownCLFrameWork() {
	if (ms_pFBO != NULL) {
		ms_memoryMananger->releaseFBO(ms_pFBOID);
		ms_pFBO = NULL;
	}
	SAFE_DELETE(ms_memoryMananger);
}


void clClass::RenderViewPortCover() {
	#ifdef useCoverQuad
		glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex2f(-1,-1);
			glTexCoord2f(1,0); glVertex2f( 1,-1);
			glTexCoord2f(1,1); glVertex2f( 1, 1);
			glTexCoord2f(0,1); glVertex2f(-1, 1);
		glEnd();
	#else
		glBegin(GL_TRIANGLES);
			glTexCoord2f(0,0); glVertex2f(-1,-1);
			glTexCoord2f(2,0); glVertex2f( 3,-1);
			glTexCoord2f(0,2); glVertex2f(-1, 3);
		glEnd();
	#endif
}

void clClass::StartCLComputation() {
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, ms_pFBO);
}

void clClass::StopCLComputation() {
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, NULL, 0);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, NULL);
}