/************************************************************
 *															*
 * decr     : Crank-Nicholson vector for the                *
 *            water surface demo	                        *
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 23.05.2005									*
 * modified	: 23.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#include "clCrNiVector.h"
#include "clCrNiVector.inl" // the GLSL Shaders

int				clCrNiVector::ms_iShaderClCrNiUser=0;

clCrNiVector::~clCrNiVector(void){
	DeleteStatic();
}

clCrNiVector::clCrNiVector(int iSizeX,int iSizeY, bool bConstant) 
{
	init(iSizeX,iSizeY, bConstant, FLOAT_TEX_R);
}

clCrNiVector::clCrNiVector(int iSize, bool bConstant) 
{
	init(iSize, bConstant, FLOAT_TEX_R);
}


void clCrNiVector::setC(float fC) {
	setSimulParam(m_fDt, fC, m_fDX, m_fDY);
}

void clCrNiVector::setSimulParam(float fDt,float fC,float fDX,float fDY) {
	m_fDt = fDt;
	m_fC  = fC;
	m_fDX = fDX;
	m_fDY = fDY;

	m_fPreFactor = (m_fDt*m_fDt*m_fC*m_fC) / (2.0f*m_fDX*m_fDY);  
}

void clCrNiVector::getSimulParam(float &fDt,float &fC,float &fDX,float &fDY) {
	fDt = m_fDt;
	fC  = m_fC;
	fDX = m_fDX;
	fDY = m_fDY;
}

void clCrNiVector::computeRHS(clUnpackedVector *cluULast, clUnpackedVector *cluUCurrent) {
	glViewport(0,0,m_memDesc.m_iWidth,m_memDesc.m_iHeight);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_pVectorTexture, 0);

		glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,cluUCurrent->m_pVectorTexture);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		glActiveTexture(GL_TEXTURE1); glBindTexture(GL_TEXTURE_2D,cluULast->m_pVectorTexture);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		ms_pShaderCrNi->Enable();
			glUniform2f(ms_f2Scalar_CrNi,1.0f/float(m_memDesc.m_iWidth),
										 1.0f/float(m_memDesc.m_iHeight));
			glUniform1f(ms_fScalar_CrNi,m_fPreFactor);
			RenderViewPortCover();
		ms_pShaderCrNi->Disable();

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glActiveTexture(GL_TEXTURE0);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
}

void clCrNiVector::InitStatic() {
	ms_iShaderClCrNiUser++;
	if (ms_iShaderClCrNiUser > 1) return;

	// load the standart shaders
	clUnpackedVector::InitStatic();

	// load the additional shader
	ms_pShaderCrNi= new CProgram(ms_vsCrNi, ms_psCrNi, TUM3D_STRING);
	ms_pShaderCrNi->Enable();
		ms_f2Scalar_CrNi = glGetUniformLocation(*ms_pShaderCrNi,"g_f2Scalar");
		ms_fScalar_CrNi  = glGetUniformLocation(*ms_pShaderCrNi,"g_fPreFac");

		ms_pShaderCrNi->SetUniformVector("g_sCurrent", 0); 
		ms_pShaderCrNi->SetUniformVector("g_sLast", 1);
	ms_pShaderCrNi->Disable();
}

void clCrNiVector::DeleteStatic() {
	ms_iShaderClCrNiUser--;
	if (ms_iShaderClCrNiUser == 0) {
		SAFE_DELETE(ms_pShaderCrNi);
	}
}
