/************************************************************
 *															*
 * decr     : A single float value stored on the GPU		*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 18.05.2005									*
 * modified	: 18.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#include "clFloat.h"
#include "clFloat.inl" // GLSL Shaders

int			clFloat::ms_iClFloatCount=0;

clFloat::~clFloat(void)
{
	DeleteStatic();
}

void clFloat::Init() 
{
	// init data for temp render-to texture
	m_TempID		= -1;
	m_pTempTexture	= NULL;

	// create a 1x1 texture
	m_iMemID = ms_memoryMananger->getTexture(clMemDescr(1,1,FLOAT_TEX_R,GL_TEXTURE_2D),m_pFloatTexture);

	// init static variables
	InitStatic();
}

void clFloat::InitStatic() {
	// load shaders
	if (ms_iClFloatCount == 0) {
		ms_pShader_Add = new CProgram(ms_vsPassThrough, ms_psAdd, TUM3D_STRING);
		ms_pShader_Add->Enable();
			ms_f2Scalar_Add=glGetUniformLocation(*ms_pShader_Add,"g_f2Scalar");
			ms_pShader_Add->SetUniformVector("g_sFloatA", 0); // set texture
			ms_pShader_Add->SetUniformVector("g_sFloatB", 1); // set texture
		ms_pShader_Add->Disable();

		ms_pShader_Mul = new CProgram(ms_vsPassThrough, ms_psMul, TUM3D_STRING);
		ms_pShader_Mul->Enable();
			ms_fScalar_Mul=glGetUniformLocation(*ms_pShader_Mul,"g_fScalar");
			ms_pShader_Mul->SetUniformVector("g_sFloatA", 0); // set texture
			ms_pShader_Mul->SetUniformVector("g_sFloatB", 1); // set texture
		ms_pShader_Mul->Disable();

		ms_pShader_Div = new CProgram(ms_vsPassThrough, ms_psDiv, TUM3D_STRING);
		ms_pShader_Div->Enable();
			ms_fScalar_Div=glGetUniformLocation(*ms_pShader_Div,"g_fScalar");
			ms_pShader_Div->SetUniformVector("g_sFloatA", 0); // set texture
			ms_pShader_Div->SetUniformVector("g_sFloatB", 1); // set texture
		ms_pShader_Div->Disable();

		ms_pShader_DivZ = new CProgram(ms_vsPassThrough, ms_psDivZ, TUM3D_STRING);
		ms_pShader_DivZ->Enable();
			ms_fScalar_DivZ=glGetUniformLocation(*ms_pShader_DivZ,"g_fScalar");
			ms_pShader_DivZ->SetUniformVector("g_sFloatA", 0); // set texture
			ms_pShader_DivZ->SetUniformVector("g_sFloatB", 1); // set texture
		ms_pShader_DivZ->Disable();

		ms_pShader_Invert = new CProgram(ms_vsPassThrough, ms_psInvert, TUM3D_STRING);
		ms_pShader_Invert->Enable();
			ms_fScalar_Invert=glGetUniformLocation(*ms_pShader_Invert,"g_fScalar");
			ms_pShader_Invert->SetUniformVector("g_sFloat", 0); // set texture
		ms_pShader_Invert->Disable();

		ms_pShader_AddS = new CProgram(ms_vsPassThrough, ms_psAddS, TUM3D_STRING);
		ms_pShader_AddS->Enable();
			ms_fScalar_AddS=glGetUniformLocation(*ms_pShader_AddS,"g_fScalar");
			ms_pShader_AddS->SetUniformVector("g_sFloat", 0); // set texture
		ms_pShader_AddS->Disable();

		ms_pShader_MulS = new CProgram(ms_vsPassThrough, ms_psMulS, TUM3D_STRING);
		ms_pShader_MulS->Enable();
			ms_fScalar_MulS=glGetUniformLocation(*ms_pShader_MulS,"g_fScalar");
			ms_pShader_MulS->SetUniformVector("g_sFloat", 0); // set texture
		ms_pShader_MulS->Disable();
	}

	// count the active incarnations of this class
	ms_iClFloatCount++;
}

void clFloat::DeleteStatic() {
	ms_iClFloatCount--;
	if (ms_iClFloatCount == 0) {
		SAFE_DELETE(ms_pShader_Add);
		SAFE_DELETE(ms_pShader_Mul);
		SAFE_DELETE(ms_pShader_Div);
		SAFE_DELETE(ms_pShader_DivZ);
		SAFE_DELETE(ms_pShader_Invert);
		SAFE_DELETE(ms_pShader_AddS);
		SAFE_DELETE(ms_pShader_MulS);
	}
}

/*******************************************************************************************
   Name:    setData
   Purpose: uploads a float to the GPU
********************************************************************************************/
void clFloat::setData(const float fVectorData) {
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D,m_pFloatTexture);
    glTexImage2D(GL_TEXTURE_2D, 0, FLOAT_TEX_R, 1, 1, 0, GL_LUMINANCE, GL_FLOAT, &fVectorData);
	glBindTexture(GL_TEXTURE_2D,0);
}


/*******************************************************************************************
   Name:    getData
   Purpose: reads back the float value from the GPU
********************************************************************************************/
void clFloat::getData(float *fVectorData) {
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_pFloatTexture, 0);
	glReadPixels(0,0,1,1,GL_LUMINANCE,GL_FLOAT,fVectorData);
}

float clFloat::getData() {
	float fValue;
	getData(&fValue);
	return fValue;
}

TCHAR* clFloat::toString() {
  float f;
  getData(&f);
  TCHAR* cpTemp   = new TCHAR[20];
  _stprintf(cpTemp,_T("%g"),f);
  return cpTemp;
}


/*******************************************************************************************
   Name:    add
   Purpose: compute target = a*this+other*b
********************************************************************************************/
void clFloat::add(clFloat *other, clFloat *target, float a, float b) {	
	Render(other,target,a,b, ms_f2Scalar_Add, ms_pShader_Add);
}

/*******************************************************************************************
   Name:    mul
   Purpose: compute target = a*this*other
********************************************************************************************/
void clFloat::mul(clFloat *other,clFloat *target, float a) {
	Render(other,target,a, ms_fScalar_Mul, ms_pShader_Mul);
}

/*******************************************************************************************
   Name:    div
   Purpose: compute target = (a*this)/(other*b)
********************************************************************************************/
void clFloat::div(clFloat *other,clFloat *target, float a, float b) {
	Render(other,target,a/b, ms_fScalar_Div, ms_pShader_Div);
}

/*******************************************************************************************
   Name:    divZ (Div Zero check)
   Purpose: compute target = (other*b == 0) ? 0 : (a*this)/(other*b) 
********************************************************************************************/
void clFloat::divZ(clFloat *other, clFloat *target, float a, float b) {
	Render(other,target, (b == 0) ? 0 : a/b, ms_fScalar_DivZ, ms_pShader_DivZ);
}

/*******************************************************************************************
   Name:    invert
   Purpose: compute target = a/(this*b)
********************************************************************************************/
void clFloat::invert(clFloat *target, float a, float b) {
	Render(target, a/b, ms_fScalar_Invert, ms_pShader_Invert);
}

/*******************************************************************************************
   Name:    add
   Purpose: compute this = this+scalar
********************************************************************************************/
void clFloat::add(float scalar, clFloat *target) {
	Render(target, scalar, ms_fScalar_AddS, ms_pShader_AddS);
}

/*******************************************************************************************
   Name:    mul
   Purpose: compute this = this*scalar
********************************************************************************************/
void clFloat::mul(float scalar,clFloat *target) {
	Render(target, scalar, ms_fScalar_MulS, ms_pShader_MulS);
}

/*******************************************************************************************
   Name:    BeginScene
   Purpose: initializes the FBO for rendering i.e. 
            getting a texture from the memman and activating it
********************************************************************************************/
void clFloat::BeginScene() {
	glViewport(0,0,1,1);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, getWriteTexture(), 0);
	checkOpenGLError("BeginScene");
}

/*******************************************************************************************
   Name:    EndScene
   Purpose: detaches the FBO and swaps the target FBO with this clFloat FBO
********************************************************************************************/
void clFloat::EndScene() {
	swapSurfaces();
	checkOpenGLError("EndScene");
}

/*******************************************************************************************
   Name:    getWriteTexture
   Purpose: retrieves a target Texture from the memman
********************************************************************************************/
GLuint clFloat::getWriteTexture() {
	m_TempID = ms_memoryMananger->getTexture(clMemDescr(1,1,FLOAT_TEX_R,GL_TEXTURE_2D),m_pTempTexture);
	return m_pTempTexture;
}

/*******************************************************************************************
   Name:    swapSurfaces
   Purpose: swaps the target texture with this clFloats texture
********************************************************************************************/
void clFloat::swapSurfaces() {
	// swap temp and vector surface and texture
	GLuint	swapTex		= m_pFloatTexture;
	m_pFloatTexture		= m_pTempTexture;
	ms_memoryMananger->releaseTexture(m_TempID,swapTex);
	m_TempID = -1;
}

void clFloat::Render(clFloat *target, float a, GLuint scalar, CProgram*	shader) {
	target->BeginScene();
		shader->Enable();
				
			glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,m_pFloatTexture);
			glUniform1f(scalar,a);

			RenderPoint();

		shader->Disable();
	target->EndScene();
}

void clFloat::Render(clFloat *other, clFloat *target, float a, GLuint scalar, CProgram*	shader) {
	target->BeginScene();
		shader->Enable();
				
			glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,m_pFloatTexture);
			glActiveTexture(GL_TEXTURE1); glBindTexture(GL_TEXTURE_2D,other->m_pFloatTexture);
			glUniform1f(scalar,a);

			RenderPoint();

		shader->Disable();
	target->EndScene();
}

void clFloat::Render(clFloat *other, clFloat *target, float a, float b, GLuint scalar, CProgram* shader) {
	target->BeginScene();
		shader->Enable();
				
			glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,m_pFloatTexture);
			glActiveTexture(GL_TEXTURE1); glBindTexture(GL_TEXTURE_2D,other->m_pFloatTexture);
			glUniform2f(scalar,a,b);

			RenderPoint();

		shader->Disable();
	target->EndScene();
}

void clFloat::RenderPoint() {
	glBegin(GL_POINTS);
		glTexCoord2f(0,0);glVertex2f(0,0);
	glEnd();
}