/************************************************************
 *															*
 * decr     : A single float value stored on the GPU		*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 18.05.2005									*
 * modified	: 18.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#pragma once

#include "clclass.h"

class clFloat :	public clClass {
public:
	clFloat()				{Init();}
	clFloat(float fValue)   {Init();setData(fValue);}
	virtual ~clFloat(void);

	virtual void Init();
	virtual void setData(const float fVectorData);
	virtual void getData(float *fVectorData);
	virtual float getData();

	virtual void add(clFloat *other, float a=1, float b=1)		{add(other,this,a,b);}
	virtual void mul(clFloat *other, float a=1)					{mul(other,this,a);}
	virtual void sub(clFloat *other, float a=1, float b=1)		{add(other,this,a,-b);}
	virtual void div(clFloat *other, float a=1, float b=1)		{div(other,this,a,b);}
	virtual void invert(float a=1, float b=1)					{invert(this,a,b);}
	virtual void add(float scalar)								{add(scalar,this);}
	virtual void mul(float scalar)								{mul(scalar,this);}	
	virtual void sub(float scalar)								{add(-scalar,this);};
	virtual void div(float scalar)								{mul(1.0f/scalar,this);};

	virtual void sub(clFloat *other, clFloat *target, float a=1, float b=1) {add(other,target,a,-b);};
	virtual void sub(float scalar, clFloat *target)							{add(target, -scalar);};
	virtual void div(float scalar, clFloat *target)							{mul(target, 1.0f/scalar);};

	virtual void add(clFloat *other, clFloat *target, float a=1, float b=1);
	virtual void mul(clFloat *other, clFloat *target, float a=1);
	virtual void div(clFloat *other, clFloat *target, float a=1, float b=1);
	virtual void divZ(clFloat *other,clFloat *target, float a=1, float b=1);
	virtual void invert(clFloat *target, float a=1, float b=1);
	virtual void add(float scalar, clFloat *target);
	virtual void mul(float scalar, clFloat *target);	

	virtual TCHAR* toString();

//	static void preOrder(int iCount) {ms_memoryMananger->preOrderTextureTarget(clMemDescr(1,1,FLOAT_TEX_R,GL_TEXTURE_2D),iCount);}

	GLuint readTexture() {return m_pFloatTexture;}

protected:
	static int			ms_iClFloatCount;
	static CProgram		*ms_pShader_Add, *ms_pShader_Mul, *ms_pShader_Div, *ms_pShader_DivZ, *ms_pShader_Invert, *ms_pShader_AddS, *ms_pShader_MulS;
	static GLint		ms_f2Scalar_Add, ms_fScalar_Mul, ms_fScalar_Div, ms_fScalar_DivZ, ms_fScalar_Invert, ms_fScalar_AddS, ms_fScalar_MulS;
	static char			*ms_psAdd, *ms_psMul, *ms_psDiv, *ms_psDivZ, *ms_psInvert, *ms_psAddS, *ms_psMulS;

	void InitStatic();
	void DeleteStatic();

	GLuint					m_pFloatTexture;
	int						m_iActiveTexture;
	int						m_iMemID;

	int						m_TempID;
	GLuint					m_pTempTexture;

	void BeginScene();
	void EndScene();
	GLuint getWriteTexture();
	void swapSurfaces();

	void RenderPoint();
	void Render(clFloat *target, float a, GLuint scalar, CProgram* shader);
	void Render(clFloat *other, clFloat *target, float a, GLuint scalar, CProgram* shader);
	void Render(clFloat *other, clFloat *target, float a, float b, GLuint scalar, CProgram* shader);

	friend class clFragmentVector; // grand access to m_pFloatTexture
};
