/************************************************************
 *															*
 * decr     : abstract base class for all fragment based	*
 *            vector clases									*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 20.05.2005									*
 * modified	: 20.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#pragma once

#include "clvector.h"
#include "clFloat.h"
#include "clMemMan.h"

struct CL_TEX2D_VERTEX {
	float x, y;
	float tu, tv;
};

class clFragmentVector : public clVector {
	public:
		GLuint	m_pVectorTexture;

		clFragmentVector(void) {}
		virtual ~clFragmentVector(void);
		virtual void init(int iSizeX, int iSizeY, bool bConstant, GLenum eFormat);
		virtual void init(int iSize, bool bConstant, GLenum eFormat);

		virtual int getSize() = NULL;
		virtual int getSizeX() = NULL;
		virtual int getSizeY() = NULL;
		virtual clMemDescr getDecription() {return m_memDesc;}

		virtual void setData(const float *fVectorData) = NULL;
		virtual void getData(float *fVectorData) = NULL;

		virtual void clear(float fClearValue=0);
		virtual float reduceAdd();
		virtual float reduceAdd(clFragmentVector* clvSecond);
		virtual void reduceAdd(clFloat *clfResult);
		virtual void reduceAdd(clFragmentVector* clvSecond,clFloat *clfResult);

		virtual void multiplyScalar(float fScalar);
		virtual void copyVector(clFragmentVector *clvSource);
		virtual void addVector(clFragmentVector* vSource, clFragmentVector* vTarget, float fScal1=1, float fScal2=1);
		virtual void subtractVector(clFragmentVector* clvSource, clFragmentVector* clvTarget, float fScal1=1, float fScal2=1) {addVector(clvSource, clvTarget, fScal1, -fScal2);};
		virtual void vectorOp(CL_enum eOpType, clFragmentVector *clvSource, clFragmentVector *clvTarget, float fScal1=1, float fScal2=1);

		virtual void addVector(clFragmentVector* vSource, clFragmentVector* vTarget, float fScal, clFloat *clfScal);
		virtual void subtractVector(clFragmentVector* vSource, clFragmentVector* vTarget, float fScal, clFloat *clfScal);

		virtual void swapSurfaces();
		virtual GLuint getReadTexture();
		clMemDescr getMemDesc() {return m_memDesc;}

		virtual TCHAR* toString(int iOutputCount);
		virtual TCHAR* toShortString(int iOutputCount);

		void BeginScene();
		void EndScene();
	protected:
		static CProgram			*ms_pShaderCopy, *ms_pShaderMulS, *ms_pShaderAdd, *ms_pShaderAddCL, *ms_pShaderSubCL,
								*ms_pShaderReduceAddFirst, *ms_pShaderReduceAddRest, *ms_pShaderReduceAddX,
								*ms_pShaderReduceAddY, *ms_pShaderCombineR, *ms_pShaderCombineRGBA, *ms_pShaderClear;
		static char				*ms_psCopy, *ms_psMulS, *ms_psAdd, *ms_psAddCL, *ms_psSubCL,
								*ms_psReduceAddFirst, *ms_psReduceAddRest, *ms_vsReduce,
								*ms_psReduceAddX, *ms_psReduceAddY, *ms_vsCombine, *ms_psCombineR,
								*ms_psCombineRGBA, *ms_psClear;
		static GLint			ms_fScalar_MulS, ms_f2Scalar_Add, ms_fScalar_AddCL, ms_fScalar_SubCL,
								ms_fScalar_ReduceStepFirst, ms_fScalar_ReduceStepRest,
								ms_fScalar_ReduceStepX, ms_fScalar_ReduceStepY, ms_fScalar_Clear, 
								ms_f2Scalar_CombineR, ms_f2Scalar_CombineRGBA;
		static int				ms_iVectorCount;

		int						m_iTempActive;
		clMemDescr				m_memDesc;
		int						m_iMemID;

		int						m_ID[2];
		GLuint					m_pTempTexture[2];

		virtual void InitStatic() = NULL;
		void InitCommonStatic();
		void DeleteCommonStatic();

		void splitQuadX(CL_TEX2D_VERTEX *hQuad);
		void splitQuadY(CL_TEX2D_VERTEX *hQuad);
		void splitQuad(CL_TEX2D_VERTEX *hQuad);
		virtual void reduceInXDirection(int &iDepth, CL_TEX2D_VERTEX *hQuad);
		virtual void reduceInYDirection(int &iDepth, CL_TEX2D_VERTEX *hQuad);
		int reduceAddInternal(bool sumFirst);
		virtual void combineLastPointsGeneral(int iDepth, GLuint pMiniTex, int iType);
		virtual void combineLastPoints(int iDepth, GLuint pMiniTex) = NULL;
		virtual float evalLastPoint(GLuint pMiniTex) = NULL;
		virtual GLuint getWriteTexture();
};
