/************************************************************
 *															*
 * decr     : Shaders for the clFragmentVector           	*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 20.05.2005									*
 * modified	: 20.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

// add clFloat Shader
CProgram*	clFragmentVector::ms_pShaderCopy=NULL;
char* clFragmentVector::ms_psCopy = "\
\
uniform sampler2D g_sOtherVec;\n\
\n\
void main(void)\n\
{\n\
	gl_FragColor = texture2D(g_sOtherVec, gl_TexCoord[0].xy);\n\
}\n";


CProgram*	clFragmentVector::ms_pShaderClear=NULL;
GLint		clFragmentVector::ms_fScalar_Clear=NULL;
char* clFragmentVector::ms_psClear = "\
\
uniform float g_fScalar;\n\
\n\
void main(void)\n\
{\n\
	gl_FragColor = vec4(g_fScalar);\n\
}\n";

// multiply scalar Shader
CProgram*	clFragmentVector::ms_pShaderMulS=NULL;
GLint		clFragmentVector::ms_fScalar_MulS=NULL;

char* clFragmentVector::ms_psMulS = "\
\
uniform sampler2D g_sVec;\n\
uniform float g_fScalar;\n\
\n\
void main(void)\n\
{\n\
	gl_FragColor = g_fScalar*texture2D(g_sVec, gl_TexCoord[0].xy);\n\
}\n";

// add vectors Shader
CProgram*	clFragmentVector::ms_pShaderAdd=NULL;
GLint		clFragmentVector::ms_f2Scalar_Add=NULL;

char* clFragmentVector::ms_psAdd = "\
\
uniform sampler2D g_sVectorA;\n\
uniform sampler2D g_sVectorB;\n\
uniform vec2 g_f2Scalar;\n\
\n\
void main(void)\n\
{\n\
	gl_FragColor = g_f2Scalar.r*texture2D(g_sVectorA, gl_TexCoord[0].xy)+g_f2Scalar.g*texture2D(g_sVectorB, gl_TexCoord[0].xy);\n\
}\n";

// add vectors Shader with clFloat
CProgram*	clFragmentVector::ms_pShaderAddCL=NULL;
GLint		clFragmentVector::ms_fScalar_AddCL=NULL;

char* clFragmentVector::ms_psAddCL = "\
\
uniform sampler2D g_sVectorA;\n\
uniform sampler2D g_sVectorB;\n\
uniform sampler2D g_sScalar;\n\
uniform float g_fScalar;\n\
\n\
void main(void)\n\
{\n\
	gl_FragColor = g_fScalar*texture2D(g_sVectorA, gl_TexCoord[0].xy)+texture2D(g_sScalar, gl_TexCoord[0].xy).x*texture2D(g_sVectorB, gl_TexCoord[0].xy);\n\
}\n";

// subtract vectors Shader with clFloat
CProgram*	clFragmentVector::ms_pShaderSubCL=NULL;
GLint		clFragmentVector::ms_fScalar_SubCL=NULL;

char* clFragmentVector::ms_psSubCL = "\
\n\
uniform sampler2D g_sVectorA;\n\
uniform sampler2D g_sVectorB;\n\
uniform sampler2D g_sScalar;\n\
uniform float g_fScalar;\n\
\n\
void main(void)\n\
{\n\
	gl_FragColor = g_fScalar*texture2D(g_sVectorA, gl_TexCoord[0].xy)-texture2D(g_sScalar, gl_TexCoord[0].xy).x*texture2D(g_sVectorB, gl_TexCoord[0].xy);\n\
}\n";


// reduce Vertex shader 
GLint clFragmentVector::ms_fScalar_ReduceStepFirst = NULL;
GLint clFragmentVector::ms_fScalar_ReduceStepRest  = NULL;
GLint clFragmentVector::ms_fScalar_ReduceStepX	   = NULL;
GLint clFragmentVector::ms_fScalar_ReduceStepY     = NULL;
char* clFragmentVector::ms_vsReduce = "\
\n\
uniform float g_fReduceStep;\n\
\n\
void main(void)\n\
{\n\
   gl_Position       = vec4( gl_Vertex.xy, 0.0, 1.0 );\n\
   gl_TexCoord[0]    = vec4( gl_MultiTexCoord0.xy, gl_MultiTexCoord0.xy + g_fReduceStep);\n\
}";

// reduce pixel shaders
CProgram*	clFragmentVector::ms_pShaderReduceAddFirst = NULL;
char* clFragmentVector::ms_psReduceAddFirst = "\n\
\n\
uniform sampler2D g_sVectorA;\n\
uniform sampler2D g_sVectorB;\n\
\n\
void main(void)\n\
{\n\
	vec4 value1A = texture2D(g_sVectorA, gl_TexCoord[0].xy);\n\
	vec4 value2A = texture2D(g_sVectorA, gl_TexCoord[0].zy);\n\
	vec4 value3A = texture2D(g_sVectorA, gl_TexCoord[0].xw);\n\
	vec4 value4A = texture2D(g_sVectorA, gl_TexCoord[0].zw);\n\
\n\
	vec4 value1B = texture2D(g_sVectorB, gl_TexCoord[0].xy);\n\
	vec4 value2B = texture2D(g_sVectorB, gl_TexCoord[0].zy);\n\
	vec4 value3B = texture2D(g_sVectorB, gl_TexCoord[0].xw);\n\
	vec4 value4B = texture2D(g_sVectorB, gl_TexCoord[0].zw);\n\
\n\
	gl_FragColor = value1A*value1B+value2A*value2B+\n\
				   value3A*value3B+value4A*value4B;\n\
}\n";

CProgram*	clFragmentVector::ms_pShaderReduceAddRest = NULL;
char* clFragmentVector::ms_psReduceAddRest = "\n\
\n\
uniform sampler2D g_sVector;\n\
\n\
void main(void)\n\
{\n\
	vec4 value1 = texture2D(g_sVector, gl_TexCoord[0].xy);\n\
	vec4 value2 = texture2D(g_sVector, gl_TexCoord[0].zy);\n\
	vec4 value3 = texture2D(g_sVector, gl_TexCoord[0].xw);\n\
	vec4 value4 = texture2D(g_sVector, gl_TexCoord[0].zw);\n\
\n\
	gl_FragColor = value1+value2+value3+value4;\n\
}\n";

CProgram*	clFragmentVector::ms_pShaderReduceAddX = NULL;
char* clFragmentVector::ms_psReduceAddX = "\n\
\n\
uniform sampler2D g_sVector;\n\
\n\
void main(void)\n\
{\n\
	vec4 value1 = texture2D(g_sVector, gl_TexCoord[0].xy);\n\
	vec4 value2 = texture2D(g_sVector, gl_TexCoord[0].zy);\n\
\n\
	gl_FragColor = value1+value2;\n\
}\n";

CProgram*	clFragmentVector::ms_pShaderReduceAddY = NULL;
char* clFragmentVector::ms_psReduceAddY = "\n\
\n\
uniform sampler2D g_sVector;\n\
\n\
void main(void)\n\
{\n\
	vec4 value1 = texture2D(g_sVector, gl_TexCoord[0].xy);\n\
	vec4 value2 = texture2D(g_sVector, gl_TexCoord[0].xw);\n\
\n\
	gl_FragColor = value1+value2;\n\
}\n";


// combine Vertex shader 
GLint clFragmentVector::ms_f2Scalar_CombineR     = NULL;
GLint clFragmentVector::ms_f2Scalar_CombineRGBA  = NULL;
char* clFragmentVector::ms_vsCombine = "\
\n\
uniform vec2 g_f2ReduceStep;\n\
\n\
void main(void)\n\
{\n\
   gl_Position       = vec4( gl_Vertex.xy, 0.0, 1.0 );\n\
   gl_TexCoord[0]    = vec4( gl_MultiTexCoord0.xy, gl_MultiTexCoord0.xy + g_f2ReduceStep.xy);\n\
}";

// combine pixel shader for R encoded vectors
CProgram*	clFragmentVector::ms_pShaderCombineR = NULL;
char* clFragmentVector::ms_psCombineR = "\n\
\n\
uniform sampler2D g_sVector;\n\
\n\
void main(void)\n\
{\n\
	vec4 value1 = texture2D(g_sVector, gl_TexCoord[0].xy);\n\
	vec4 value2 = texture2D(g_sVector, gl_TexCoord[0].zy);\n\
	vec4 value3 = texture2D(g_sVector, gl_TexCoord[0].xw);\n\
	vec4 value4 = texture2D(g_sVector, gl_TexCoord[0].zw);\n\
\n\
	gl_FragColor = value1+value2+value3+value4;\n\
}\n";

// combine pixel shader for RGBA encoded vectors
CProgram*	clFragmentVector::ms_pShaderCombineRGBA= NULL;
char* clFragmentVector::ms_psCombineRGBA= "\n\
\n\
uniform sampler2D g_sVector;\n\
\n\
void main(void)\n\
{\n\
	vec4 value1 = texture2D(g_sVector, gl_TexCoord[0].xy);\n\
	vec4 value2 = texture2D(g_sVector, gl_TexCoord[0].zy);\n\
	vec4 value3 = texture2D(g_sVector, gl_TexCoord[0].xw);\n\
	vec4 value4 = texture2D(g_sVector, gl_TexCoord[0].zw);\n\
\n\
   gl_FragColor = vec4(dot(vec4(1.0,1.0,1.0,1.0),value1+value2+value3+value4));\n\
}\n";
