/************************************************************
 *															*
 * decr     : memory mananger for the cl framework			*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 18.05.2005									*
 * modified	: 18.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#include "clMemMan.h"


clMemMan::clMemMan() {
	// nothing todo yet
}

clMemMan::~clMemMan(void) {
	for (unsigned int i = 0; i<m_vTextureList.size();++i) {
		#ifdef _DEBUG
		if (m_vTextureList[i].lock) {VERB("Warning: releasing unfreed texture\n")};
		#endif
		m_vTextureList[i].ReleaseData();
	}
	m_vTextureList.clear();

	for (unsigned int i = 0; i<m_vFBOList.size();++i) {
		#ifdef _DEBUG
		if (m_vFBOList[i].lock) {VERB("Warning: releasing unfreed FBO\n")};
		#endif
		m_vFBOList[i].ReleaseData();
	}
	m_vFBOList.clear();

}

int clMemMan::getFBO(GLuint &pFBO) {
	int ID = findFirstFBO();
	if (ID < 0) ID = genNewFBO();

	pFBO		= m_vFBOList[ID].pFBO;
	m_vFBOList[ID].lock = true;
	return ID;
}

int clMemMan::getTexture(clMemDescr memDesc, GLuint &pTexture) {
	int ID = findFirstTexture(memDesc);
	if (ID < 0) ID = genNewTexture(memDesc);

	pTexture	= m_vTextureList[ID].pTexture;
	m_vTextureList[ID].lock = true;
	return ID;
}

void clMemMan::releaseFBO(unsigned int ID) {
	assert(ID < m_vFBOList.size());
	m_vFBOList[ID].lock = false;
}

void clMemMan::releaseFBO(unsigned int ID, GLuint &pFBO) {
	assert(ID < m_vFBOList.size());

	m_vFBOList[ID].pFBO		= pFBO;
	m_vFBOList[ID].lock		= false;
}

void clMemMan::releaseTexture(unsigned int ID) {
	assert(ID < m_vTextureList.size());
	m_vTextureList[ID].lock = false;
}

void clMemMan::releaseTexture(unsigned int ID, GLuint &pTexture) {
	assert(ID < m_vTextureList.size());

	m_vTextureList[ID].pTexture = pTexture;
	m_vTextureList[ID].lock		= false;
}

int clMemMan::findFirstFBO() {
	unsigned int i;

	for (i = 0; i<m_vFBOList.size();++i)
		if (!m_vFBOList[i].lock) break;

	return (i < m_vFBOList.size()) ? i : -1;
}

int clMemMan::genNewFBO() {
	FBOVecElem newFBO;
	newFBO.CreateData();
	m_vFBOList.push_back(newFBO);
	return int(m_vFBOList.size())-1;
}

int clMemMan::findFirstTexture(const clMemDescr &memDesc) {
	unsigned int i;

	for (i = 0; i<m_vTextureList.size();++i)
		if ( m_vTextureList[i].desc == memDesc && !m_vTextureList[i].lock) break;

	return (i < m_vTextureList.size()) ? i : -1;
}

int clMemMan::genNewTexture(const clMemDescr &memDesc) {
	TexVecElem newTex(memDesc);
	newTex.CreateData();
	m_vTextureList.push_back(newTex);
	return int(m_vTextureList.size())-1;
}