/************************************************************
 *															*
 * decr     : memory mananger for the cl framework			*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 18.05.2005									*
 * modified	: 18.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#pragma once

#include <tools/openGLUtils.h>
#include <vector>

class clMemDescr {
	public :
		int m_iWidth;
		int m_iHeight;
		GLenum m_eFormat;
		GLenum m_eTarget;

		clMemDescr() : m_iWidth(0), m_iHeight(0), m_eFormat(GL_RGBA8), m_eTarget(GL_TEXTURE_2D) {};
		clMemDescr(int iWidth, int iHeight,	GLenum eFormat, GLenum eTarget) : m_iWidth(iWidth), m_iHeight(iHeight), m_eFormat(eFormat), m_eTarget(eTarget) {}
		clMemDescr(const clMemDescr& other) : m_iWidth(other.m_iWidth), m_iHeight(other.m_iHeight), m_eFormat(other.m_eFormat), m_eTarget(other.m_eTarget) {}

		bool operator == ( const clMemDescr& other ) const {
			return (other.m_iWidth==m_iWidth && 
				    other.m_iHeight==m_iHeight && 
					other.m_eFormat==m_eFormat &&
					other.m_eTarget == m_eTarget); 
		}
};


class VecElem {
public :
	bool lock;	
};

class FBOVecElem : public VecElem {
public:
	GLuint pFBO;

	FBOVecElem()		{ pFBO=NULL; lock=false; }
	FBOVecElem(bool l)	{ pFBO=NULL; lock=l;}
	FBOVecElem(const FBOVecElem &other) {lock=other.lock; pFBO=other.pFBO; }

	void CreateData() {
		ReleaseData();
	    glGenFramebuffersEXT(1, &pFBO);

		checkOpenGLError("CreateData Memman:create FBO");
	}

	void ReleaseData() {
		if (pFBO != NULL) {
			glDeleteRenderbuffersEXT(1,&pFBO);
			pFBO = NULL;
		}
	}

	bool creationPending() {return pFBO == NULL;}

};

class TexVecElem : public VecElem {
public:
	clMemDescr desc;
	GLuint pTexture;

	TexVecElem()						{ pTexture=NULL; lock=false; }
	TexVecElem(clMemDescr d)			{ pTexture=NULL; lock=false; desc=d; }
	TexVecElem(clMemDescr d, bool l)	{ pTexture=NULL; lock=l; desc=d; }
	TexVecElem(const TexVecElem &other) { desc=other.desc; lock=other.lock; pTexture=other.pTexture; }
	void CreateData() {
		ReleaseData();

	    glGenTextures(1, &pTexture);
		glBindTexture(desc.m_eTarget, pTexture);
		glTexParameterf(desc.m_eTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameterf(desc.m_eTarget, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameterf(desc.m_eTarget, GL_TEXTURE_WRAP_S, GL_REPEAT/*GL_CLAMP_TO_EDGE*/);
		glTexParameterf(desc.m_eTarget, GL_TEXTURE_WRAP_T, GL_REPEAT/*GL_CLAMP_TO_EDGE*/);
		glTexImage2D(desc.m_eTarget, 0, desc.m_eFormat, desc.m_iWidth, desc.m_iHeight, 0, GL_RGBA, GL_FLOAT, NULL);
		
		checkOpenGLError("CreateData Memman:create texture");

		glBindTexture(desc.m_eTarget, NULL);
	}

	void ReleaseData() {
		if (pTexture != NULL) {
			glDeleteTextures(1,&pTexture);
			pTexture = NULL;
		}
	}

	bool creationPending() {return pTexture == NULL;}

};

typedef std::vector<FBOVecElem> FBOVEC;
typedef std::vector<TexVecElem> TEXVEC;

class clMemMan {
	public:
		clMemMan();
		virtual ~clMemMan(void);

		int getTexture(clMemDescr memDesc, GLuint &pTexture);
		void releaseTexture(unsigned int ID);
		void releaseTexture(unsigned int ID, GLuint &pTexture);

		int getFBO(GLuint &pFBO);
		void releaseFBO(unsigned int ID);
		void releaseFBO(unsigned int ID, GLuint &pFBO);

		static bool CheckFramebufferStatus() {
			GLenum status;
			status = (GLenum) glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
			switch(status) {
				case GL_FRAMEBUFFER_COMPLETE_EXT:
					return true;
					break;
				case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
					VERB("Unsupported framebuffer format\n");
					break;
				case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT:
					VERB("Framebuffer incomplete, missing attachment\n");
					break;
				case GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT:
					VERB("Framebuffer incomplete, duplicate attachment\n");
					break;
				case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT:
					VERB("Framebuffer incomplete, attached images must have same dimensions\n");
					break;
				case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT:
					VERB("Framebuffer incomplete, attached images must have same format\n");
					break;
				case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT:
					VERB("Framebuffer incomplete, missing draw buffer\n");
					break;
				case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT:
					VERB("Framebuffer incomplete, missing read buffer\n");
					break;
				case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:
					VERB("Framebuffer incomplete attachment\n");
					break;
				default:
					VERB("Unknown framebuffer error\n");
			}
			return false;
		}

		FBOVEC	m_vFBOList;
	protected:
//		FBOVEC	m_vFBOList;
		TEXVEC	m_vTextureList;

		int findFirstFBO();
		int genNewFBO();

		int findFirstTexture(const clMemDescr &memDesc);
		int genNewTexture(const clMemDescr &memDesc);


};