/************************************************************
 *															*
 * decr     : RGBA ecnoded vector class						*
 * version  : 1.0a											*
 * author   : Jens Krger									*
 * date     : 20.05.2005									*
 * modified	: 31.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#include "clFragmentVector.h"
#include "clPackedVector.h"
#include "clPackedVector.inl" // GLSL Shaders

int clPackedVector::ms_iPVectorCount = 0;

/*******************************************************************************************
   Name:    default destructor
   Purpose: destroyes this object
********************************************************************************************/
clPackedVector::~clPackedVector(void)	{
	DeleteStatic();	
}

void clPackedVector::DeleteStatic() {
	ms_iPVectorCount --;
	if (ms_iPVectorCount == 0) {
		DeleteCommonStatic();

		SAFE_DELETE(ms_pShaderPack);
		SAFE_DELETE(ms_pShaderUnpack);

		for (int i = 0;i<4;i++) {
			SAFE_DELETE(ms_pShaderMultiplyMat_noadd[i]);
			SAFE_DELETE(ms_pShaderMultiplyMat[i]);
		}

		SAFE_DELETE(ms_pShaderMultiplyMatNoShift_noadd);
		SAFE_DELETE(ms_pShaderMultiplyMatNoShift);
	}
}

/*******************************************************************************************
   Name:    InitStatic
   Purpose: loads all shaders required for this vector, if loaded already use the old one
********************************************************************************************/
void clPackedVector::InitStatic() {
	ms_iPVectorCount++;
	if (ms_iPVectorCount > 1) return;

	InitCommonStatic();

	ms_pShaderPack= new CProgram(ms_vsPassThrough, ms_psPack, TUM3D_STRING);
	ms_pShaderPack->Enable();
		ms_f2Scalar_Pack=glGetUniformLocation(*ms_pShaderPack,"g_f2Scalar");
		ms_f4Scalar_Pack=glGetUniformLocation(*ms_pShaderPack,"g_f4Scalar");
		ms_pShaderPack->SetUniformVector("g_sVector", 0); // set texture
	ms_pShaderPack->Disable();

	ms_pShaderUnpack= new CProgram(ms_vsPassThrough, ms_psUnpack, TUM3D_STRING);
	ms_pShaderUnpack->Enable();
		ms_f2Scalar_Unpack=glGetUniformLocation(*ms_pShaderUnpack,"g_f2Scalar");
		ms_pShaderUnpack->SetUniformVector("g_sVector", 0); // set texture
	ms_pShaderUnpack->Disable();

	for (int i = 0;i<4;i++) {
		ms_pShaderMultiplyMat_noadd[i]= new CProgram(ms_vsPassThrough, ms_psMultiplyMat_noadd[i], TUM3D_STRING);
		ms_pShaderMultiplyMat_noadd[i]->Enable();
			ms_f4Scalar_MultiplyMat_noadd[i]=glGetUniformLocation(*(ms_pShaderMultiplyMat_noadd[i]),"g_f4Scalar");
			ms_pShaderMultiplyMat_noadd[i]->SetUniformVector("g_sVector", 0); 
			ms_pShaderMultiplyMat_noadd[i]->SetUniformVector("g_sVector2", 1);
		ms_pShaderMultiplyMat_noadd[i]->Disable();

		ms_pShaderMultiplyMat[i]= new CProgram(ms_vsPassThrough, ms_psMultiplyMat[i], TUM3D_STRING);
		ms_pShaderMultiplyMat[i]->Enable();
			ms_f4Scalar_MultiplyMat[i]=glGetUniformLocation(*ms_pShaderMultiplyMat[i],"g_f4Scalar");
			ms_pShaderMultiplyMat[i]->SetUniformVector("g_sVector",  0); 
			ms_pShaderMultiplyMat[i]->SetUniformVector("g_sVector2", 1);
			ms_pShaderMultiplyMat[i]->SetUniformVector("g_sVector3", 2);
		ms_pShaderMultiplyMat[i]->Disable();
	}

	ms_pShaderMultiplyMatNoShift_noadd= new CProgram(ms_vsPassThrough, ms_psMultiplyMatNoShift_noadd, TUM3D_STRING);
	ms_pShaderMultiplyMatNoShift_noadd->Enable();
		ms_fScalar_MultiplyMatNoShift_noadd=glGetUniformLocation(*ms_pShaderMultiplyMatNoShift_noadd,"g_fScalar");
		ms_pShaderMultiplyMatNoShift_noadd->SetUniformVector("g_sVector", 0); 
		ms_pShaderMultiplyMatNoShift_noadd->SetUniformVector("g_sVector2", 1);
	ms_pShaderMultiplyMatNoShift_noadd->Disable();

	ms_pShaderMultiplyMatNoShift= new CProgram(ms_vsPassThrough, ms_psMultiplyMatNoShift, TUM3D_STRING);
	ms_pShaderMultiplyMatNoShift->Enable();
		ms_fScalar_MultiplyMatNoShift=glGetUniformLocation(*ms_pShaderMultiplyMatNoShift,"g_fScalar");
		ms_pShaderMultiplyMatNoShift->SetUniformVector("g_sVector",  0); 
		ms_pShaderMultiplyMatNoShift->SetUniformVector("g_sVector2", 1);
		ms_pShaderMultiplyMatNoShift->SetUniformVector("g_sVector3", 2);
	ms_pShaderMultiplyMatNoShift->Disable();
}
	
/*******************************************************************************************
   Name:    multiplyVector
   Purpose: computes a vector vector product while shifting the second vector:
	          vTarget[i] = fScalar*this[i]*vSource[i+iShift*4+iSubShift]
********************************************************************************************/
void clPackedVector::multiplyVector(clPackedVector* vSource, clPackedVector* vTarget, float fScalar, signed int iShift, int iSubShift) {
	vTarget->BeginScene();
		glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,this->m_pVectorTexture);
		glActiveTexture(GL_TEXTURE1); glBindTexture(GL_TEXTURE_2D,vSource->m_pVectorTexture);

		if (iShift==0 && iSubShift==0) {
			ms_pShaderMultiplyMatNoShift_noadd->Enable();
				glUniform1f(ms_fScalar_MultiplyMatNoShift_noadd,fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMatNoShift_noadd->Disable();
		} else {
			ms_pShaderMultiplyMat_noadd[iSubShift]->Enable();
				glUniform4f(ms_f4Scalar_MultiplyMat_noadd[iSubShift],m_memDesc.m_iWidth,m_memDesc.m_iHeight, float(iShift), fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMat_noadd[iSubShift]->Disable();
		}
	vTarget->EndScene();
}

/*******************************************************************************************
   Name:    multiplyAddVector
   Purpose: computes a vector vector product while shifting the second vector:
	          vTarget[i] = vTarget[i]+(fScalar*this[i]*vSource[i+iShift*4+iSubShift])
********************************************************************************************/
void clPackedVector::multiplyAddVector(clPackedVector* vSource, clPackedVector* vTarget, float fScalar, signed int iShift, int iSubShift) {
	vTarget->BeginScene();
		glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,this->m_pVectorTexture);
		glActiveTexture(GL_TEXTURE1); glBindTexture(GL_TEXTURE_2D,vSource->m_pVectorTexture);
		glActiveTexture(GL_TEXTURE2); glBindTexture(GL_TEXTURE_2D,vTarget->m_pVectorTexture);

		if (iShift==0 && iSubShift==0) {
			ms_pShaderMultiplyMatNoShift->Enable();
				glUniform1f(ms_fScalar_MultiplyMatNoShift,fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMatNoShift->Disable();
		} else {
			ms_pShaderMultiplyMat[iSubShift]->Enable();
				glUniform4f(ms_f4Scalar_MultiplyMat[iSubShift],m_memDesc.m_iWidth,m_memDesc.m_iHeight, float(iShift), fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMat[iSubShift]->Disable();
		}
	vTarget->EndScene();

}

/*******************************************************************************************
   Name:    setData
   Purpose: uploads a vector to the GPU
********************************************************************************************/
void clPackedVector::setData(const float *fVectorData) {
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D,m_pVectorTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, m_memDesc.m_eFormat, m_memDesc.m_iWidth, m_memDesc.m_iHeight, 0, GL_RGBA, GL_FLOAT, fVectorData);
	glBindTexture(GL_TEXTURE_2D,0);
}


/*******************************************************************************************
   Name:    getData
   Purpose: reads back the vector values from the GPU
********************************************************************************************/
void clPackedVector::getData(float *fVectorData) {
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_pVectorTexture, 0);
		glReadPixels(0,0,m_memDesc.m_iWidth,m_memDesc.m_iHeight,GL_RGBA,GL_FLOAT, fVectorData);
}

///*******************************************************************************************
//   Name:    evalLastPoint
//   Purpose: transfers the last point via back to the main mem
//********************************************************************************************/
float clPackedVector::evalLastPoint(GLuint pMiniText) {
	float fData[4];
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, pMiniText, 0);
		glReadPixels(0,0,1,1,GL_RGBA,GL_FLOAT,fData);
	return fData[0]+fData[1]+fData[2]+fData[3];
}

/*******************************************************************************************
   Name:    unpack
   Purpose: convert the packed vector into an unpacked representation
********************************************************************************************/
void clPackedVector::unpack(clUnpackedVector *vTarget) {
	glViewport(0,0,vTarget->m_memDesc.m_iWidth,vTarget->m_memDesc.m_iHeight);

	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, vTarget->m_pVectorTexture, 0); // write directly to texture to avoid swap
		ms_pShaderUnpack->Enable();
			glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,m_pVectorTexture);
			glUniform2f(ms_f2Scalar_Unpack,m_memDesc.m_iWidth,m_memDesc.m_iHeight);
			RenderViewPortCover();
		ms_pShaderUnpack->Disable();
}

/*******************************************************************************************
   Name:    repack
   Purpose: read the data of the unpacked vector vTarget into this packed vector
********************************************************************************************/
void clPackedVector::repack(clUnpackedVector *vSource) {
	FLOATVECTOR4 vConstants = FLOATVECTOR4((float)m_memDesc.m_iWidth*2.0f,
										   (float)m_memDesc.m_iHeight*2.0f,
										   (float)m_memDesc.m_iWidth/2.0f,
										    0.5f/(float)m_memDesc.m_iWidth);

	glViewport(0,0,m_memDesc.m_iWidth,m_memDesc.m_iHeight);

	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_pVectorTexture, 0); // write directly to texture to avoid swap
		
		ms_pShaderPack->Enable();
			glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,vSource->m_pVectorTexture);
			glUniform4f(ms_f4Scalar_Pack, vConstants.x,vConstants.y,vConstants.z,vConstants.w);
			glUniform2f(ms_f2Scalar_Pack,float(m_memDesc.m_iWidth), float(m_memDesc.m_iHeight));

			RenderViewPortCover();
		ms_pShaderPack->Disable();

}