/************************************************************
 *															*
 * decr     : Shaders for the clPackedVector           	    *
 * version  : 1.0a											*
 * author   : Jens Krger									*
 * date     : 20.05.2005									*
 * modified	: 31.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

// pack Shader
CProgram*	clPackedVector::ms_pShaderPack = NULL;
GLint		clPackedVector::ms_f4Scalar_Pack = NULL;
GLint		clPackedVector::ms_f2Scalar_Pack = NULL;
char* clPackedVector::ms_psPack = "\
\
uniform sampler2D g_sVector;\n\
uniform vec4 g_f4Scalar;\n\
uniform vec2 g_f2Scalar;\n\
\n\
void main(void)\n\
{\n\
	vec4 result;\n\
	vec2 intCoords = floor(gl_TexCoord[0].xy*g_f2Scalar);\n\
	vec2 coords = vec2(  mod(intCoords.x,g_f4Scalar.z)*4.0, floor(intCoords.x/g_f4Scalar.z) + intCoords.y*2.0 );\n\
\n\
	coords /= g_f4Scalar.xy;\n\
\n\
	vec4 vecValue = texture2D(g_sVector,coords);\n\
\n\
	result.r = texture2D(g_sVector,coords).r;	coords.x += g_f4Scalar.w;\n\
	result.g = texture2D(g_sVector,coords).r;	coords.x += g_f4Scalar.w;\n\
	result.b = texture2D(g_sVector,coords).r;	coords.x += g_f4Scalar.w;\n\
	result.a = texture2D(g_sVector,coords).r;\n\
\n\
	gl_FragColor = result;\n\
}\n";

// unpack Shader
CProgram*	clPackedVector::ms_pShaderUnpack = NULL;
GLint		clPackedVector::ms_f2Scalar_Unpack = NULL;
char* clPackedVector::ms_psUnpack = "\
\
uniform sampler2D g_sVector;\n\
uniform vec2	  g_f2Scalar;\n\
\n\
void main(void)\n\
{\n\
	vec4 result;\n\
	vec2 coords = vec2(gl_TexCoord[0].x/2.0,gl_TexCoord[0].y);\n\
\n\
	if (floor( mod((gl_TexCoord[0].y*g_f2Scalar.y*2.0),2.0) )==1.0) coords.x += 0.5;\n\
\n\
	vec4 vecValue = texture2D(g_sVector,coords);\n\
\n\
	if (floor( mod((gl_TexCoord[0].x*g_f2Scalar.x*2.0),4.0) ) == 3.0)\n\
		result = vec4(vecValue.a,0,0,0);\n\
	else if (floor( mod((gl_TexCoord[0].x*g_f2Scalar.x*2.0),4.0) ) == 2.0)\n\
		result = vec4(vecValue.b,0,0,0);\n\
	else if (floor( mod((gl_TexCoord[0].x*g_f2Scalar.x*2.0),4.0) ) == 1.0)\n\
		result = vec4(vecValue.g,0,0,0);\n\
	else\n\
		result = vec4(vecValue.r,0,0,0);\n\
\n\
	gl_FragColor = result;\n\
}\n";

CProgram*	clPackedVector::ms_pShaderMultiplyMatNoShift_noadd = NULL;
GLint		clPackedVector::ms_fScalar_MultiplyMatNoShift_noadd = NULL;
char* clPackedVector::ms_psMultiplyMatNoShift_noadd = "\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform float	  g_fScalar;\n\
\n\
void main() {\n\
	gl_FragColor = texture2D(g_sVector,gl_TexCoord[0].xy)*texture2D(g_sVector2,gl_TexCoord[0].xy)*g_fScalar;\n\
}";

// Shifted Vector-Vector Multiply
CProgram    *clPackedVector::ms_pShaderMultiplyMat_noadd[4]  = {NULL, NULL, NULL, NULL};
GLint		clPackedVector::ms_f4Scalar_MultiplyMat_noadd[4] = {NULL, NULL, NULL, NULL};
char* clPackedVector::ms_psMultiplyMat_noadd[4] = {
// Subshift = 0
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec2 computeShiftedCoords(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	float temp = f2InCoords.x*texSize.x+fShift;\n\
	vec2  f2ShiftCoords;\n\
\n\
    f2ShiftCoords.x = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f2ShiftCoords.y = floor(temp);\n\
\n\
    f2ShiftCoords.y = f2InCoords.y*texSize.y+f2ShiftCoords.y;\n\
\n\
	return f2ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec2 f2ShiftCoords = computeShiftedCoords(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy);\n\
	gl_FragColor = texture2D(g_sVector,gl_TexCoord[0].xy)*texture2D(g_sVector2,f2ShiftCoords/g_f4Scalar.xy)*g_f4Scalar.w;\n\
}",
// Subshift = 1
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec4 computeShiftedCoordsP1(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	vec2 temp;\n\
	temp.x = f2InCoords.x*texSize.x+fShift;\n\
	temp.y = temp.x+1.0;\n\
\n\
	vec4 f4ShiftCoords;\n\
\n\
    f4ShiftCoords.xz = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f4ShiftCoords.yw = f2InCoords.y*texSize.y+floor(temp);\n\
	return f4ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec4 f4ShiftCoords = computeShiftedCoordsP1(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy)/g_f4Scalar.xyxy;\n\
\n\
	vec4 f4Result;\n\
	vec4 f4MatVector = texture2D(g_sVector,gl_TexCoord[0].xy)*g_f4Scalar.w;\n\
\n\
	f4Result.rgb = f4MatVector.rgb * texture2D(g_sVector2,f4ShiftCoords.xy).gba;\n\
	f4Result.a   = f4MatVector.a   * texture2D(g_sVector2,f4ShiftCoords.zw).r;\n\
\n\
	gl_FragColor = f4Result;\n\
}", 
// Subshift = 2
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec4 computeShiftedCoordsP1(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	vec2 temp;\n\
	temp.x = f2InCoords.x*texSize.x+fShift;\n\
	temp.y = temp.x+1.0;\n\
\n\
	vec4 f4ShiftCoords;\n\
\n\
    f4ShiftCoords.xz = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f4ShiftCoords.yw = f2InCoords.y*texSize.y+floor(temp);\n\
	return f4ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec4 f4ShiftCoords = computeShiftedCoordsP1(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy)/g_f4Scalar.xyxy;\n\
\n\
	vec4 f4Result;\n\
	vec4 f4MatVector = texture2D(g_sVector,gl_TexCoord[0].xy)*g_f4Scalar.w;\n\
\n\
	f4Result.rg = f4MatVector.rg * texture2D(g_sVector2,f4ShiftCoords.xy).ba;\n\
	f4Result.ba = f4MatVector.ba * texture2D(g_sVector2,f4ShiftCoords.zw).rg;\n\
\n\
	gl_FragColor = f4Result;\n\
}", 
// Subshift = 3
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec4 computeShiftedCoordsP1(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	vec2 temp;\n\
	temp.x = f2InCoords.x*texSize.x+fShift;\n\
	temp.y = temp.x+1.0;\n\
\n\
	vec4 f4ShiftCoords;\n\
\n\
    f4ShiftCoords.xz = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f4ShiftCoords.yw = f2InCoords.y*texSize.y+floor(temp);\n\
	return f4ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec4 f4ShiftCoords = computeShiftedCoordsP1(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy)/g_f4Scalar.xyxy;\n\
\n\
	vec4 f4Result;\n\
	vec4 f4MatVector = texture2D(g_sVector,gl_TexCoord[0].xy)*g_f4Scalar.w;\n\
\n\
	f4Result.r   = f4MatVector.r   * texture2D(g_sVector2,f4ShiftCoords.xy).a;\n\
	f4Result.gba = f4MatVector.gba * texture2D(g_sVector2,f4ShiftCoords.zw).rgb;\n\
\n\
	gl_FragColor = f4Result;\n\
}"};


CProgram*	clPackedVector::ms_pShaderMultiplyMatNoShift = NULL;
GLint		clPackedVector::ms_fScalar_MultiplyMatNoShift = NULL;
char* clPackedVector::ms_psMultiplyMatNoShift = "\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform sampler2D g_sVector3;\n\
uniform float	  g_fScalar;\n\
\n\
void main() {\n\
	gl_FragColor = texture2D(g_sVector3,gl_TexCoord[0].xy)+texture2D(g_sVector,gl_TexCoord[0].xy)*texture2D(g_sVector2,gl_TexCoord[0].xy)*g_fScalar;\n\
}";

CProgram*	clPackedVector::ms_pShaderMultiplyMat[4] = {NULL, NULL, NULL, NULL};
GLint		clPackedVector::ms_f4Scalar_MultiplyMat[4] = {NULL, NULL, NULL, NULL};
char* clPackedVector::ms_psMultiplyMat[4] = {
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform sampler2D g_sVector3;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec2 computeShiftedCoords(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	float  temp = f2InCoords.x*texSize.x+fShift;\n\
	vec2 f2ShiftCoords;\n\
\n\
    f2ShiftCoords.x = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f2ShiftCoords.y = floor(temp);\n\
\n\
    f2ShiftCoords.y = f2InCoords.y*texSize.y+f2ShiftCoords.y;\n\
\n\
	return f2ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec2 f2ShiftCoords = computeShiftedCoords(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy);\n\
	gl_FragColor = texture2D(g_sVector3,gl_TexCoord[0].xy)+texture2D(g_sVector,gl_TexCoord[0].xy)*texture2D(g_sVector2,f2ShiftCoords/g_f4Scalar.xy)*g_f4Scalar.w;\n\
}",
// Subshift = 1
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform sampler2D g_sVector3;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec4 computeShiftedCoordsP1(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	vec2 temp;\n\
	temp.x = f2InCoords.x*texSize.x+fShift;\n\
	temp.y = temp.x+1.0;\n\
\n\
	vec4 f4ShiftCoords;\n\
\n\
    f4ShiftCoords.xz = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f4ShiftCoords.yw = f2InCoords.y*texSize.y+floor(temp);\n\
	return f4ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec4 f4ShiftCoords = computeShiftedCoordsP1(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy)/g_f4Scalar.xyxy;\n\
\n\
	vec4 f4Result = texture2D(g_sVector3,gl_TexCoord[0].xy);\n\
	vec4 f4MatVector = texture2D(g_sVector,gl_TexCoord[0].xy)*g_f4Scalar.w;\n\
\n\
	f4Result.rgb += f4MatVector.rgb * texture2D(g_sVector2,f4ShiftCoords.xy).gba;\n\
	f4Result.a   += f4MatVector.a   * texture2D(g_sVector2,f4ShiftCoords.zw).r;\n\
\n\
	gl_FragColor = f4Result;\n\
}", 
// Subshift = 2
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform sampler2D g_sVector3;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec4 computeShiftedCoordsP1(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	vec2 temp;\n\
	temp.x = f2InCoords.x*texSize.x+fShift;\n\
	temp.y = temp.x+1.0;\n\
\n\
	vec4 f4ShiftCoords;\n\
\n\
    f4ShiftCoords.xz = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f4ShiftCoords.yw = f2InCoords.y*texSize.y+floor(temp);\n\
	return f4ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec4 f4ShiftCoords = computeShiftedCoordsP1(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy)/g_f4Scalar.xyxy;\n\
\n\
	vec4 f4Result = texture2D(g_sVector3,gl_TexCoord[0].xy);\n\
	vec4 f4MatVector = texture2D(g_sVector,gl_TexCoord[0].xy)*g_f4Scalar.w;\n\
\n\
	f4Result.rg += f4MatVector.rg * texture2D(g_sVector2,f4ShiftCoords.xy).ba;\n\
	f4Result.ba += f4MatVector.ba * texture2D(g_sVector2,f4ShiftCoords.zw).rg;\n\
\n\
	gl_FragColor = f4Result;\n\
}", 
// Subshift = 3
"\
\n\
uniform sampler2D g_sVector;\n\
uniform sampler2D g_sVector2;\n\
uniform sampler2D g_sVector3;\n\
uniform vec4	  g_f4Scalar;\n\
\n\
vec4 computeShiftedCoordsP1(vec2 f2InCoords, float fShift, vec2 texSize) {\n\
	vec2 temp;\n\
	temp.x = f2InCoords.x*texSize.x+fShift;\n\
	temp.y = temp.x+1.0;\n\
\n\
	vec4 f4ShiftCoords;\n\
\n\
    f4ShiftCoords.xz = mod(temp, texSize.x); \n\
    temp = temp / texSize.x;\n\
    f4ShiftCoords.yw = f2InCoords.y*texSize.y+floor(temp);\n\
	return f4ShiftCoords;\n\
}\n\
\n\
void main() {\n\
	vec4 f4ShiftCoords = computeShiftedCoordsP1(gl_TexCoord[0].xy,g_f4Scalar.z, g_f4Scalar.xy)/g_f4Scalar.xyxy;\n\
\n\
	vec4 f4Result = texture2D(g_sVector3,gl_TexCoord[0].xy);\n\
	vec4 f4MatVector = texture2D(g_sVector,gl_TexCoord[0].xy)*g_f4Scalar.w;\n\
\n\
	f4Result.r   += f4MatVector.r   * texture2D(g_sVector2,f4ShiftCoords.xy).a;\n\
	f4Result.gba += f4MatVector.gba * texture2D(g_sVector2,f4ShiftCoords.zw).rgb;\n\
\n\
	gl_FragColor = f4Result;\n\
}"};
