/************************************************************
 *															*
 * decr     : R encoded (non packed) vector class			*
 * version  : 1.0											*
 * author   : Jens Krger									*
 * date     : 20.05.2005									*
 * modified	: 20.05.2005									*
 * e-mail   : jens.krueger@in.tum.de						*
 *															*
 ************************************************************/

#include "clFragmentVector.h"
#include "clUnpackedVector.h"
#include "clUnpackedVector.inl" // the GLSL Shader

int clUnpackedVector::ms_iUPVectorCount = 0;

clUnpackedVector::~clUnpackedVector(void) {
	DeleteStatic();
}

void clUnpackedVector::DeleteStatic() {
	ms_iUPVectorCount--;
	if (ms_iUPVectorCount == 0) {
		DeleteCommonStatic();

		SAFE_DELETE(ms_pShaderMultiplyMat_noadd);
		SAFE_DELETE(ms_pShaderMultiplyMat);
		SAFE_DELETE(ms_pShaderMultiplyMatNoShift_noadd);
		SAFE_DELETE(ms_pShaderMultiplyMatNoShift);
	}
}

/*******************************************************************************************
   Name:    InitStatic
   Purpose: loads all shaders required for this vector, if loaded already use the old one
********************************************************************************************/
void clUnpackedVector::InitStatic() {
	ms_iUPVectorCount++;
	if (ms_iUPVectorCount > 1) return;

	InitCommonStatic();

	ms_pShaderMultiplyMat_noadd= new CProgram(ms_vsPassThrough, ms_psMultiplyMat_noadd, TUM3D_STRING);
	ms_pShaderMultiplyMat_noadd->Enable();
		ms_f4Scalar_MultiplyMat_noadd=glGetUniformLocation(*ms_pShaderMultiplyMat_noadd,"g_f4Scalar");
		ms_pShaderMultiplyMat_noadd->SetUniformVector("g_sVector", 0); 
		ms_pShaderMultiplyMat_noadd->SetUniformVector("g_sVector2", 1);
	ms_pShaderMultiplyMat_noadd->Disable();

	ms_pShaderMultiplyMat= new CProgram(ms_vsPassThrough, ms_psMultiplyMat, TUM3D_STRING);
	ms_pShaderMultiplyMat->Enable();
		ms_f4Scalar_MultiplyMat=glGetUniformLocation(*ms_pShaderMultiplyMat,"g_f4Scalar");
		ms_pShaderMultiplyMat->SetUniformVector("g_sVector",  0); 
		ms_pShaderMultiplyMat->SetUniformVector("g_sVector2", 1);
		ms_pShaderMultiplyMat->SetUniformVector("g_sVector3", 2);
	ms_pShaderMultiplyMat->Disable();

	ms_pShaderMultiplyMatNoShift_noadd= new CProgram(ms_vsPassThrough, ms_psMultiplyMatNoShift_noadd, TUM3D_STRING);
	ms_pShaderMultiplyMatNoShift_noadd->Enable();
		ms_fScalar_MultiplyMatNoShift_noadd=glGetUniformLocation(*ms_pShaderMultiplyMatNoShift_noadd,"g_fScalar");
		ms_pShaderMultiplyMatNoShift_noadd->SetUniformVector("g_sVector", 0); 
		ms_pShaderMultiplyMatNoShift_noadd->SetUniformVector("g_sVector2", 1);
	ms_pShaderMultiplyMatNoShift_noadd->Disable();

	ms_pShaderMultiplyMatNoShift= new CProgram(ms_vsPassThrough, ms_psMultiplyMatNoShift, TUM3D_STRING);
	ms_pShaderMultiplyMatNoShift->Enable();
		ms_fScalar_MultiplyMatNoShift=glGetUniformLocation(*ms_pShaderMultiplyMatNoShift,"g_fScalar");
		ms_pShaderMultiplyMatNoShift->SetUniformVector("g_sVector",  0); 
		ms_pShaderMultiplyMatNoShift->SetUniformVector("g_sVector2", 1);
		ms_pShaderMultiplyMatNoShift->SetUniformVector("g_sVector3", 2);
	ms_pShaderMultiplyMatNoShift->Disable();
}

/*******************************************************************************************
   Name:    multiplyVector
   Purpose: computes a vector vector product while shifting the second vector:
	          vTarget[i] = fScalar*this[i]*vSource[i+iShift*4+iSubShift]
********************************************************************************************/
void clUnpackedVector::multiplyVector(clUnpackedVector* vSource, clUnpackedVector* vTarget, float fScalar, signed int iShift) {
	vTarget->BeginScene();
		glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,this->m_pVectorTexture);
		glActiveTexture(GL_TEXTURE1); glBindTexture(GL_TEXTURE_2D,vSource->m_pVectorTexture);

		if (iShift==0) {
			ms_pShaderMultiplyMatNoShift_noadd->Enable();
				glUniform1f(ms_fScalar_MultiplyMatNoShift_noadd,fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMatNoShift_noadd->Disable();
		} else {
			ms_pShaderMultiplyMat_noadd->Enable();
				glUniform4f(ms_f4Scalar_MultiplyMat_noadd,m_memDesc.m_iWidth,m_memDesc.m_iHeight, float(iShift), fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMat_noadd->Disable();
		}

	vTarget->EndScene();
}

/*******************************************************************************************
   Name:    multiplyAddVector
   Purpose: computes a vector vector product while shifting the second vector:
	          vTarget[i] = vTarget[i]+(fScalar*this[i]*vSource[i+iShift*4+iSubShift])
********************************************************************************************/
void clUnpackedVector::multiplyAddVector(clUnpackedVector* vSource, clUnpackedVector* vTarget, float fScalar, signed int iShift) {
	vTarget->BeginScene();
		glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,this->m_pVectorTexture);
		glActiveTexture(GL_TEXTURE1); glBindTexture(GL_TEXTURE_2D,vSource->m_pVectorTexture);
		glActiveTexture(GL_TEXTURE2); glBindTexture(GL_TEXTURE_2D,vTarget->m_pVectorTexture);

		if (iShift==0) {
			ms_pShaderMultiplyMatNoShift->Enable();
				glUniform1f(ms_fScalar_MultiplyMatNoShift,fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMatNoShift->Disable();
		} else {
			ms_pShaderMultiplyMat->Enable();
				glUniform4f(ms_f4Scalar_MultiplyMat,m_memDesc.m_iWidth,m_memDesc.m_iHeight, float(iShift), fScalar);
				RenderViewPortCover();
			ms_pShaderMultiplyMat->Disable();
		}
	vTarget->EndScene();
}

///*******************************************************************************************
//   Name:    evalLastPoint
//   Purpose: transfers the last point via back to the main mem
//********************************************************************************************/
float clUnpackedVector::evalLastPoint(GLuint pMiniFBO) {
	float fData;
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, pMiniFBO);
		glReadPixels(0,0,1,1,GL_LUMINANCE,GL_FLOAT,&fData);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	return fData;
}


/*******************************************************************************************
   Name:    setData
   Purpose: uploads a vector to the GPU
********************************************************************************************/
void clUnpackedVector::setData(const float *fVectorData) {
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D,m_pVectorTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, m_memDesc.m_eFormat, m_memDesc.m_iWidth,m_memDesc.m_iHeight, 0, GL_LUMINANCE, GL_FLOAT, fVectorData);
	glBindTexture(GL_TEXTURE_2D,0);
}


/*******************************************************************************************
   Name:    getData
   Purpose: reads back the vector values from the GPU
********************************************************************************************/
void clUnpackedVector::getData(float *fVectorData) {
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, m_pVectorTexture, 0);
		glReadPixels(0,0,m_memDesc.m_iWidth,m_memDesc.m_iHeight,GL_LUMINANCE,GL_FLOAT, fVectorData);
}