/**********************************************************
 *                                                        *
 * program: OpenGL clFramework Test Application           *
 * author : Jens Krger                                   * 
 * date   : 18.May.2005                                   *
 * e-mail : jens.krueger@in.tum.de                        *
 *                                                        *
 **********************************************************/

#include "main.h"

int main(int argc, char **argv) {
    glutInit(&argc,argv);
    glutInitWindowSize(g_iWinWidth,g_iWinHeight);
    glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGBA|GLUT_ALPHA|GLUT_DEPTH);
	glutCreateWindow(PROGNAME.c_str());

	// Register Callback-functions
	glutDisplayFunc(glutDisplayCallback);
    glutReshapeFunc(glutReshapeCallback); 
    glutIdleFunc(glutIdleCallback);

    glutKeyboardFunc(glutKeyboardCallback);
	glutSpecialFunc(glutSpecialKeyboardCallback);

	glutMouseFunc(glutMouseCallback);
	glutMotionFunc(glutMouseMotionCallback);

	Initialize();
	PrintHelp();

	glutMainLoop();
	return(0);        // never reached but suppresses warning message
}

void ClearSimulation() {
	clClass::StartCLComputation();

	g_cluULast->clear();
#ifdef useUnpacked
	g_cluUNext->clear();
#else
	g_clUNext->clear();
#endif
	g_cluUCurrent->clear();

	clClass::StopCLComputation();
}

void Initialize() {
	g_iGridSizeX = 512;
	g_iGridSizeY = 512;

	g_fC= 0.3f;
	g_fDt = 1.0f;	g_fDX = 1.0f;	g_fDY = 1.0f;

	if (!clClass::StartupCLFrameWork()) {
		printf("CLFramework failed to initalize.\n");
		exit(-1);
	}

	g_cluRHS      = new clCrNiVector(g_iGridSizeX,g_iGridSizeY);
	g_cluRHS->setSimulParam(g_fDt,g_fC,g_fDX,g_fDY);

	#ifdef useUnpacked
		clCrNiMatrix *cnMatrix = new clCrNiMatrix(g_iGridSizeX,g_iGridSizeY,g_fDt,g_fC,g_fDX,g_fDY);
		g_cluUNext  = new clUnpackedVector(g_iGridSizeX,g_iGridSizeY); g_cluUNext->clear();
		g_pCGSolver = new clCGSolver(cnMatrix,g_cluUNext,g_cluRHS,CL_UNPACKED);
	#else
		clPackedCrNiMatrix *cnMatrix = new clPackedCrNiMatrix(g_iGridSizeX,g_iGridSizeY,g_fDt,g_fC,g_fDX,g_fDY);
		g_clUNext   = new clPackedVector(g_iGridSizeX,g_iGridSizeY); g_clUNext->clear();
		g_clRHS     = new clPackedVector(g_iGridSizeX,g_iGridSizeY);
		g_pCGSolver = new clCGSolver(cnMatrix,g_clUNext,g_clRHS,CL_PACKED);
	#endif

	g_cluULast    = new clUnpackedVector(g_iGridSizeX,g_iGridSizeY); g_cluULast->clear();
	g_cluUCurrent = new clUnpackedVector(g_iGridSizeX,g_iGridSizeY); g_cluUCurrent->clear();

	g_pMainShader = new CProgram(g_vsMain,g_psMain,TUM3D_STRING);
	g_pMainShader->Enable();
		g_pMainShader->SetUniformVector("g_f2StepSize", 1.0f/float(g_iGridSizeX), 1.0f/float(g_iGridSizeY));
		g_pMainShader->SetUniformVector("g_sVector", 0);
	g_pMainShader->Disable();

	g_pMainDropShader= new CProgram(g_vsMainDrop,g_psMainDrop,TUM3D_STRING);
}

void DeleteData() {
 	SAFE_DELETE( g_pCGSolver ); 	// deleting the solver implitly deletes "g_clUNext" and "g_clRHS"
#ifndef useUnpacked
	SAFE_DELETE( g_cluRHS );
#endif
	SAFE_DELETE( g_cluUCurrent );
	SAFE_DELETE( g_cluULast );

	clClass::ShutdownCLFrameWork();
	SAFE_DELETE(g_pMainShader);
	SAFE_DELETE(g_pMainDropShader);
}

void glutKeyboardCallback(unsigned char ucKey, int iXPos,int iYPos) {
	switch(ucKey) {
		case 'q': case 'Q': case 27:
			DeleteData();
			exit(0);
			break;
		case ' ':
			g_bRain = !g_bRain;
			break;
	}
}

void glutSpecialKeyboardCallback(int iKey,int iXPos,int iYPos) {
	switch(iKey) {
		case GLUT_KEY_F1: PrintHelp(); break;
	}	
}

void glutReshapeCallback(int iWinWidth, int iWinHeight) {
    g_iWinWidth  = iWinWidth;
    g_iWinHeight = iWinHeight;
    g_fAspect	 = float(iWinWidth)/float(iWinHeight);

    glViewport(0,0,g_iWinWidth,g_iWinHeight);
    glMatrixMode(GL_PROJECTION);		
    glLoadIdentity();
    gluPerspective(60.0,g_fAspect,0.1,100.0);  
    glMatrixMode(GL_MODELVIEW);

    glutPostRedisplay();
}

void glutIdleCallback() {
	glutPostRedisplay();
}

void glutDisplayCallback() {
	// startup the frame
	openGLUtils::fpsTimerStartFrame();

	clClass::StartCLComputation();

#ifdef useUnpacked
	g_cluRHS->computeRHS(g_cluULast, g_cluUCurrent);
	g_iSteps = g_pCGSolver->solveNT(2);
	g_cluULast->copyVector(g_cluUCurrent);
	g_cluUCurrent->copyVector(g_cluUNext);
#else
	g_cluRHS->computeRHS(g_cluULast, g_cluUCurrent);
	g_clRHS->repack(g_cluRHS);
	g_iSteps = g_pCGSolver->solveNT(2);
	g_cluULast->copyVector(g_cluUCurrent);
	g_clUNext->unpack(g_cluUCurrent);
#endif

	clClass::StopCLComputation();

    glViewport(0,0,g_iWinWidth,g_iWinHeight);

	glActiveTexture(GL_TEXTURE0); glBindTexture(GL_TEXTURE_2D,g_cluUCurrent->m_pVectorTexture);

	g_pMainShader->Enable();
		glBegin(GL_QUADS);
			glTexCoord2f(0,0);	glVertex3f(-1,-1,0);
			glTexCoord2f(0,1);  glVertex3f(-1,1,0);
			glTexCoord2f(1,1);  glVertex3f(1,1,0);
			glTexCoord2f(1,0);  glVertex3f(1,-1,0);
		glEnd();
	g_pMainShader->Disable();

	if (g_bRain) {
		static int counter = 0;
		if (!(counter % 5)) {
			float posX = float(rand())/float(RAND_MAX);
			float posY = float(rand())/float(RAND_MAX);
			SetDrop(posX,posY);
		}
		counter++;
	}

	if (g_bMouseButtonDown[GLUT_LEFT_BUTTON] > 0) SetDrop(float(g_iXMousePos)/float(g_iWinWidth),1-float(g_iYMousePos)/float(g_iWinHeight));

	glDisable(GL_TEXTURE_2D);

	glutSwapBuffers();
	openGLUtils::fpsTimerShowFPS(PROGNAME.c_str());
}

void SetDrop(float fXPos, float fYPos) {
	glViewport(0,0,g_cluUCurrent->getDecription().m_iWidth,g_cluUCurrent->getDecription().m_iHeight);

	clClass::StartCLComputation();

	// hack, directly write to read surface
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, g_cluUCurrent->getReadTexture(), 0);

	g_pMainDropShader->Enable();
		glBegin(GL_POINTS);
			glVertex2f((fXPos-0.5f)*2.0f,(fYPos-0.5f)*2.0f);
		glEnd();
	g_pMainDropShader->Disable();

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);

	clClass::StopCLComputation();
}

void glutMouseCallback(int iButton,int iState,int iXPos,int iYPos) {
	g_bMouseButtonDown[iButton] = (iState==GLUT_DOWN) ? 1 : 0;
    
    if (g_bMouseButtonDown[GLUT_RIGHT_BUTTON] > 0) ClearSimulation();

	g_iXMousePos = iXPos;
	g_iYMousePos = iYPos;
}

void glutMouseMotionCallback(int iXPos,int iYPos) {
    if (g_bMouseButtonDown[GLUT_LEFT_BUTTON] > 0) {
        g_bMouseButtonDown[GLUT_LEFT_BUTTON]=2;
    }
    if (g_bMouseButtonDown[GLUT_RIGHT_BUTTON] > 0) {
		g_bMouseButtonDown[GLUT_RIGHT_BUTTON]=2;
	}
	glutPostRedisplay();

	g_iXMousePos = iXPos;
	g_iYMousePos = iYPos;
}


void PrintHelp() {
	VERBLN;
	VERBST(PROGNAME.c_str());VERBLN;
	VERB("==================================");VERBLN;
	VERBLN;
	VERB("   q, ESC : Quit");VERBLN;
	VERB("   F1     : print this help text");VERBLN;
}


