/**********************************************************
 *                                                        *
 * program: OpenGL clFramework Test Application           *
 * author : Jens Krger                                   * 
 * date   : 18.May.2005                                   *
 * e-mail : jens.krueger@in.tum.de                        *
 *                                                        *
 **********************************************************/

#include "Framework/clCGSolver.h"
#include "Framework/clCrNiVector.h"

#ifdef useUnpacked
	#include "Framework/clCrNiMatrix.h"
#else
	#include "Framework/clPackedCrNiMatrix.h"
#endif

using namespace std;

string			PROGNAME				= "clFramework Water Surface Demo";
int				g_iWinWidth				= 800;
int				g_iWinHeight			= 600;
float			g_fAspect				= 1.0f;
int 			g_bMouseButtonDown[3]	= {0, 0, 0};
bool			g_bRain					= false; 
int 			g_iXMousePos			= 0;
int 			g_iYMousePos			= 0;

// cg-solver variables
FLOAT*					g_pfRHSData;
int						g_iGridSizeX, g_iGridSizeY;
clCGSolver*				g_pCGSolver;
clUnpackedVector*		g_cluUCurrent;
clUnpackedVector*		g_cluULast;
clCrNiVector*			g_cluRHS;
#ifdef useUnpacked
	clUnpackedVector*		g_cluUNext;
	#define timeVec1 g_cluRHS
	#define timeVec2 g_cluUNext
#else
	clPackedVector*			g_clUNext;
	clPackedVector*			g_clRHS;
	#define timeVec1 g_clRHS
	#define timeVec2 g_clUNext
#endif

float					g_fDt, g_fC, g_fDX, g_fDY;
int						g_iSteps;

bool					g_bTimeclPerformance;
CProgram				*g_pMainShader, *g_pMainDropShader;

char *g_vsMainDrop = "\
void main(void)\
{\
	gl_Position       = vec4( gl_Vertex.xy, 0.0, 1.0 );\
	gl_TexCoord[0].xy = gl_MultiTexCoord0.xy;\
}";

char *g_psMainDrop = "\
uniform sampler2D g_sVector;\n\
\n\
void main(void)\n\
{\n\
gl_FragColor = vec4(5.0);\n\
}";

char *g_vsMain = "\
\n\
uniform vec2 g_f2StepSize;\n\
\n\
void main(void)\
{\
	gl_Position       = vec4( gl_Vertex.xy, 0.0, 1.0 );\
\n\
	gl_TexCoord[0] = vec4(gl_MultiTexCoord0.x, gl_MultiTexCoord0.y - g_f2StepSize.y, gl_MultiTexCoord0.y + g_f2StepSize.y, 0);\n\
	gl_TexCoord[1] = vec4(gl_MultiTexCoord0.y, gl_MultiTexCoord0.x - g_f2StepSize.x, gl_MultiTexCoord0.x + g_f2StepSize.x, 0);\n\
}";

char *g_psMain = "\
\n\
uniform sampler2D g_sVector;\n\
\n\
void main(void){\n\
\n\
	vec3 viewPos      = vec3(0.0,1.0,0.0);\n\
	vec3 lightPos     = normalize(vec3( 0.2,  1.0,  0.5 ));\n\
	vec3 mtrlAmbient  = vec3( 0.0,  0.0,  0.0 );\n\
	vec3 mtrlDiffuse  = vec3( 0.4,  0.4,  0.4 );\n\
	vec3 mtrlSpecular = vec3( 1.0,  1.0,  1.0 );\n\
\n\
	vec4 f4Neigbours = vec4(texture2D(g_sVector, gl_TexCoord[0].xy).r,\n\
				   	        texture2D(g_sVector, gl_TexCoord[0].xz).r,\n\
					        texture2D(g_sVector, gl_TexCoord[1].yx).r,\n\
					        texture2D(g_sVector, gl_TexCoord[1].zx).r);\n\
\n\
	vec3 normal = normalize(vec3(f4Neigbours.x-f4Neigbours.y,2.0,f4Neigbours.z-f4Neigbours.w));\n\
\n\
	float fHeight        = texture2D(g_sVector, vec2(gl_TexCoord[0].x,gl_TexCoord[1].x)).r;\n\
\n\
	vec2 uv = gl_TexCoord[0].xy-0.5;\n\
	vec3 lightDir = normalize(lightPos-vec3(uv.x, fHeight, uv.y));\n\
\n\
	vec3 halfVec        = normalize(lightPos + viewPos);\n\
\n\
	vec3 color;\n\
	color  = mtrlAmbient;\n\
    color += mtrlDiffuse * clamp(dot(lightDir, normal), 0.0, 1.0);\n\
	color += mtrlSpecular * pow(clamp(dot(halfVec, normal), 0.0, 1.0),32.0);\n\
\n\
	gl_FragColor = vec4(color,1.0);\n\
}";

// GLUT Callbacks
void glutKeyboardCallback(unsigned char ucKey, int iXPos,int iYPos);
void glutSpecialKeyboardCallback(int iKey, int iXPos,int iYPos);
void glutReshapeCallback(int iWinWidth, int iWinHeight);
void glutIdleCallback();
void glutDisplayCallback();
void glutMouseCallback(int iButton, int iState, int iXPos, int iYPos);
void glutMouseMotionCallback(int x, int y);

// Helpers
void Initialize();
void DeleteData();
void PrintHelp();

// Simulation
void ClearSimulation();
void SetDrop(float fXPos, float fYPos);

// C++ Main method
int main(int argc, char **argv);